@@ -103,6 +103,8 @@
  * Last reviewed on 2006-08-11 (0.10.10)
  */
 
+#include "gstconfig.h"
+
 #include "gst_private.h"
 #include <stdlib.h>
 #include <stdio.h>
@@ -128,34 +130,41 @@
 #define GST_PLUGIN_SEPARATOR ","
 
 static gboolean gst_initialized = FALSE;
+static gboolean gst_deinitialized = FALSE;
 
 #ifndef GST_DISABLE_REGISTRY
 static GList *plugin_paths = NULL;      /* for delayed processing in post_init */
 #endif
 
-extern gint _gst_trace_on;
+#ifndef GST_DISABLE_GST_DEBUG
+extern const gchar *priv_gst_dump_dot_dir;
+#endif
 
 /* defaults */
-#ifdef HAVE_FORK
+#if defined(HAVE_FORK) && !defined(GST_HAVE_UNSAFE_FORK)
 #define DEFAULT_FORK TRUE
 #else
 #define DEFAULT_FORK FALSE
 #endif /* HAVE_FORK */
 
-/* set to TRUE when segfaults need to be left as is, FIXME, this variable is
- * global. */
-gboolean _gst_disable_segtrap = FALSE;
+/* set to TRUE when segfaults need to be left as is */
+static gboolean _gst_disable_segtrap = FALSE;
 
 /* control the behaviour of registry rebuild */
 static gboolean _gst_enable_registry_fork = DEFAULT_FORK;
 
+/*set to TRUE when registry needn't to be updated */
+static gboolean _gst_disable_registry_update = FALSE;
+
 static void load_plugin_func (gpointer data, gpointer user_data);
 static gboolean init_pre (GOptionContext * context, GOptionGroup * group,
     gpointer data, GError ** error);
 static gboolean init_post (GOptionContext * context, GOptionGroup * group,
     gpointer data, GError ** error);
+#ifndef GST_DISABLE_OPTION_PARSING
 static gboolean parse_goption_arg (const gchar * s_opt,
     const gchar * arg, gpointer data, GError ** err);
+#endif
 
 static GSList *preload_plugins = NULL;
 
@@ -186,6 +195,7 @@ enum
   ARG_PLUGIN_PATH,
   ARG_PLUGIN_LOAD,
   ARG_SEGTRAP_DISABLE,
+  ARG_REGISTRY_UPDATE_DISABLE,
   ARG_REGISTRY_FORK_DISABLE
 };
 
@@ -292,8 +302,9 @@ parse_debug_list (const gchar * list)
 GOptionGroup *
 gst_init_get_option_group (void)
 {
+#ifndef GST_DISABLE_OPTION_PARSING
   GOptionGroup *group;
-  const static GOptionEntry gst_args[] = {
+  static const GOptionEntry gst_args[] = {
     {"gst-version", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
         (gpointer) parse_goption_arg, N_("Print the GStreamer version"), NULL},
     {"gst-fatal-warnings", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
@@ -335,6 +346,11 @@ gst_init_get_option_group (void)
           (gpointer) parse_goption_arg,
           N_("Disable trapping of segmentation faults during plugin loading"),
         NULL},
+    {"gst-disable-registry-update", 0, G_OPTION_FLAG_NO_ARG,
+          G_OPTION_ARG_CALLBACK,
+          (gpointer) parse_goption_arg,
+          N_("Disable updating the registry"),
+        NULL},
     {"gst-disable-registry-fork", 0, G_OPTION_FLAG_NO_ARG,
           G_OPTION_ARG_CALLBACK,
           (gpointer) parse_goption_arg,
@@ -368,6 +384,9 @@ gst_init_get_option_group (void)
   g_option_group_set_translation_domain (group, GETTEXT_PACKAGE);
 
   return group;
+#else
+  return NULL;
+#endif
 }
 
 /**
@@ -392,26 +411,31 @@ gst_init_get_option_group (void)
 gboolean
 gst_init_check (int *argc, char **argv[], GError ** err)
 {
+#ifndef GST_DISABLE_OPTION_PARSING
   GOptionGroup *group;
   GOptionContext *ctx;
+#endif
   gboolean res;
 
   if (!g_thread_supported ())
     g_thread_init (NULL);
 
-  GST_INFO ("initializing GStreamer");
-
   if (gst_initialized) {
     GST_DEBUG ("already initialized gst");
     return TRUE;
   }
-
+#ifndef GST_DISABLE_OPTION_PARSING
   ctx = g_option_context_new ("- GStreamer initialization");
   g_option_context_set_ignore_unknown_options (ctx, TRUE);
   group = gst_init_get_option_group ();
   g_option_context_add_group (ctx, group);
   res = g_option_context_parse (ctx, argc, argv, err);
   g_option_context_free (ctx);
+#else
+  init_pre (NULL, NULL, NULL, NULL);
+  init_post (NULL, NULL, NULL, NULL);
+  res = TRUE;
+#endif
 
   gst_initialized = res;
 
@@ -471,11 +495,13 @@ add_path_func (gpointer data, gpointer user_data)
 }
 #endif
 
+#ifndef GST_DISABLE_OPTION_PARSING
 static void
 prepare_for_load_plugin_func (gpointer data, gpointer user_data)
 {
   preload_plugins = g_slist_prepend (preload_plugins, g_strdup (data));
 }
+#endif
 
 static void
 load_plugin_func (gpointer data, gpointer user_data)
@@ -495,7 +521,7 @@ load_plugin_func (gpointer data, gpointer user_data)
   } else {
     if (err) {
       /* Report error to user, and free error */
-      GST_ERROR ("Failed to load plugin: %s\n", err->message);
+      GST_ERROR ("Failed to load plugin: %s", err->message);
       g_error_free (err);
     } else {
       GST_WARNING ("Failed to load plugin: \"%s\"", filename);
@@ -503,6 +529,7 @@ load_plugin_func (gpointer data, gpointer user_data)
   }
 }
 
+#ifndef GST_DISABLE_OPTION_PARSING
 static void
 split_and_iterate (const gchar * stringlist, gchar * separator, GFunc iterator,
     gpointer user_data)
@@ -527,6 +554,7 @@ split_and_iterate (const gchar * stringlist, gchar * separator, GFunc iterator,
     g_strfreev (strings);
   }
 }
+#endif
 
 /* we have no fail cases yet, but maybe in the future */
 static gboolean
@@ -557,6 +585,7 @@ init_pre (GOptionContext * context, GOptionGroup * group, gpointer data,
 #ifdef ENABLE_NLS
   setlocale (LC_ALL, "");
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 #endif /* ENABLE_NLS */
 
 #ifndef GST_DISABLE_GST_DEBUG
@@ -571,6 +600,8 @@ init_pre (GOptionContext * context, GOptionGroup * group, gpointer data,
       parse_debug_list (debug_list);
     }
   }
+
+  priv_gst_dump_dot_dir = g_getenv ("GST_DEBUG_DUMP_DOT_DIR");
 #endif
   /* This is the earliest we can make stuff show up in the logs.
    * So give some useful info about GStreamer here */
@@ -607,23 +638,15 @@ gst_register_core_elements (GstPlugin * plugin)
   return TRUE;
 }
 
-static GstPluginDesc plugin_desc = {
-  GST_VERSION_MAJOR,
-  GST_VERSION_MINOR,
-  "staticelements",
-  "core elements linked into the GStreamer library",
-  gst_register_core_elements,
-  VERSION,
-  GST_LICENSE,
-  PACKAGE,
-  GST_PACKAGE_NAME,
-  GST_PACKAGE_ORIGIN,
-
-  GST_PADDING_INIT
-};
-
 #ifndef GST_DISABLE_REGISTRY
 
+typedef enum
+{
+  REGISTRY_SCAN_AND_UPDATE_FAILURE = 0,
+  REGISTRY_SCAN_AND_UPDATE_SUCCESS_NOT_CHANGED,
+  REGISTRY_SCAN_AND_UPDATE_SUCCESS_UPDATED
+} GstRegistryScanAndUpdateResult;
+
 /*
  * scan_and_update_registry:
  * @default_registry: the #GstRegistry
@@ -632,9 +655,12 @@ static GstPluginDesc plugin_desc = {
  *
  * Scans for registry changes and eventually updates the registry cache. 
  *
- * Return: %TRUE if the registry could be updated
+ * Return: %REGISTRY_SCAN_AND_UPDATE_FAILURE if the registry could not scanned
+ *         or updated, %REGISTRY_SCAN_AND_UPDATE_SUCCESS_NOT_CHANGED if the
+ *         registry is clean and %REGISTRY_SCAN_AND_UPDATE_SUCCESS_UPDATED if
+ *         it has been updated and the cache needs to be re-read.
  */
-static gboolean
+static GstRegistryScanAndUpdateResult
 scan_and_update_registry (GstRegistry * default_registry,
     const gchar * registry_file, gboolean write_changes, GError ** error)
 {
@@ -642,18 +668,17 @@ scan_and_update_registry (GstRegistry * default_registry,
   gboolean changed = FALSE;
   GList *l;
 
-  GST_DEBUG ("reading registry cache: %s", registry_file);
-#ifdef USE_BINARY_REGISTRY
-  gst_registry_binary_read_cache (default_registry, registry_file);
-#else
-  gst_registry_xml_read_cache (default_registry, registry_file);
-#endif
+  GST_INFO ("Validating registry cache: %s", registry_file);
+  /* It sounds tempting to just compare the mtime of directories with the mtime
+   * of the registry cache, but it does not work. It would not catch updated
+   * plugins, which might bring more or less features.
+   */
+
   /* scan paths specified via --gst-plugin-path */
   GST_DEBUG ("scanning paths added via --gst-plugin-path");
   for (l = plugin_paths; l != NULL; l = l->next) {
     GST_INFO ("Scanning plugin path: \"%s\"", (gchar *) l->data);
-    /* CHECKME: add changed |= here as well? */
-    gst_registry_scan_path (default_registry, (gchar *) l->data);
+    changed |= gst_registry_scan_path (default_registry, (gchar *) l->data);
   }
   /* keep plugin_paths around in case a re-scan is forced later on */
 
@@ -687,10 +712,12 @@ scan_and_update_registry (GstRegistry * default_registry,
      * system-installed ones */
     home_plugins = g_build_filename (g_get_home_dir (),
         ".gstreamer-" GST_MAJORMINOR, "plugins", NULL);
+    GST_DEBUG ("scanning home plugins %s", home_plugins);
     changed |= gst_registry_scan_path (default_registry, home_plugins);
     g_free (home_plugins);
 
     /* add the main (installed) library path */
+    GST_DEBUG ("scanning main plugins %s", PLUGINDIR);
     changed |= gst_registry_scan_path (default_registry, PLUGINDIR);
   } else {
     gchar **list;
@@ -709,12 +736,12 @@ scan_and_update_registry (GstRegistry * default_registry,
 
   if (!changed) {
     GST_INFO ("Registry cache has not changed");
-    return TRUE;
+    return REGISTRY_SCAN_AND_UPDATE_SUCCESS_NOT_CHANGED;
   }
 
   if (!write_changes) {
-    GST_INFO ("Registry cached changed, but writing is disabled. Not writing.");
-    return TRUE;
+    GST_INFO ("Registry cache changed, but writing is disabled. Not writing.");
+    return REGISTRY_SCAN_AND_UPDATE_FAILURE;
   }
 
   GST_INFO ("Registry cache changed. Writing new registry cache");
@@ -726,11 +753,11 @@ scan_and_update_registry (GstRegistry * default_registry,
     g_set_error (error, GST_CORE_ERROR, GST_CORE_ERROR_FAILED,
         _("Error writing registry cache to %s: %s"),
         registry_file, g_strerror (errno));
-    return FALSE;
+    return REGISTRY_SCAN_AND_UPDATE_FAILURE;
   }
 
   GST_INFO ("Registry cache written successfully");
-  return TRUE;
+  return REGISTRY_SCAN_AND_UPDATE_SUCCESS_UPDATED;
 }
 
 static gboolean
@@ -775,27 +802,25 @@ ensure_current_registry_forking (GstRegistry * default_registry,
   }
 
   if (pid == 0) {
-    gboolean res;
-    gchar res_byte;
+    gint result_code;
 
     /* this is the child. Close the read pipe */
     (void) close (pfd[0]);
 
     GST_DEBUG ("child reading registry cache");
-    res =
+    result_code =
         scan_and_update_registry (default_registry, registry_file, TRUE, NULL);
 
     /* need to use _exit, so that any exit handlers registered don't
      * bring down the main program */
-    GST_DEBUG ("child exiting: %s", (res) ? "SUCCESS" : "FAILURE");
+    GST_DEBUG ("child exiting: %d", result_code);
 
     /* make valgrind happy (yes, you can call it insane) */
     g_free ((char *) registry_file);
 
     /* write a result byte to the pipe */
-    res_byte = res ? '1' : '0';
     do {
-      ret = write (pfd[1], &res_byte, 1);
+      ret = write (pfd[1], &result_code, sizeof (result_code));
     } while (ret == -1 && errno == EINTR);
     /* if ret == -1 now, we could not write to pipe, probably 
      * means parent has exited before us */
@@ -803,7 +828,7 @@ ensure_current_registry_forking (GstRegistry * default_registry,
 
     _exit (0);
   } else {
-    gchar res_byte;
+    gint result_code;
 
     /* parent. Close write pipe */
     (void) close (pfd[1]);
@@ -811,7 +836,7 @@ ensure_current_registry_forking (GstRegistry * default_registry,
     /* Wait for result from the pipe */
     GST_DEBUG ("Waiting for data from child");
     do {
-      ret = read (pfd[0], &res_byte, 1);
+      ret = read (pfd[0], &result_code, sizeof (result_code));
     } while (ret == -1 && errno == EINTR);
 
     if (ret == -1) {
@@ -835,14 +860,15 @@ ensure_current_registry_forking (GstRegistry * default_registry,
       return FALSE;
     }
 
-    if (res_byte == '1') {
+    if (result_code == REGISTRY_SCAN_AND_UPDATE_SUCCESS_UPDATED) {
       GST_DEBUG ("Child succeeded. Parent reading registry cache");
+      _priv_gst_registry_remove_cache_plugins (default_registry);
 #ifdef USE_BINARY_REGISTRY
       gst_registry_binary_read_cache (default_registry, registry_file);
 #else
       gst_registry_xml_read_cache (default_registry, registry_file);
 #endif
-    } else {
+    } else if (result_code == REGISTRY_SCAN_AND_UPDATE_FAILURE) {
       GST_DEBUG ("Child failed. Parent re-scanning registry, ignoring errors.");
       scan_and_update_registry (default_registry, registry_file, FALSE, NULL);
     }
@@ -854,10 +880,11 @@ ensure_current_registry_forking (GstRegistry * default_registry,
 static gboolean
 ensure_current_registry (GError ** error)
 {
-  char *registry_file;
+  gchar *registry_file;
   GstRegistry *default_registry;
-  gboolean ret;
+  gboolean ret = TRUE;
   gboolean do_fork;
+  gboolean do_update;
 
   default_registry = gst_registry_get_default ();
   registry_file = g_strdup (g_getenv ("GST_REGISTRY"));
@@ -871,30 +898,50 @@ ensure_current_registry (GError ** error)
 #endif
   }
 
-  /* first see if forking is enabled */
-  do_fork = _gst_enable_registry_fork;
-  if (do_fork) {
-    const gchar *fork_env;
+  GST_INFO ("reading registry cache: %s", registry_file);
+#ifdef USE_BINARY_REGISTRY
+  gst_registry_binary_read_cache (default_registry, registry_file);
+#else
+  gst_registry_xml_read_cache (default_registry, registry_file);
+#endif
 
-    /* forking enabled, see if it is disabled with an env var */
-    if ((fork_env = g_getenv ("GST_REGISTRY_FORK"))) {
-      /* fork enabled for any value different from "no" */
-      do_fork = strcmp (fork_env, "no") != 0;
+  do_update = !_gst_disable_registry_update;
+  if (do_update) {
+    const gchar *update_env;
+
+    if ((update_env = g_getenv ("GST_REGISTRY_UPDATE"))) {
+      /* do update for any value different from "no" */
+      do_update = (strcmp (update_env, "no") != 0);
     }
   }
 
-  /* now check registry with or without forking */
-  if (do_fork) {
-    GST_DEBUG ("forking for registry rebuild");
-    ret = ensure_current_registry_forking (default_registry, registry_file,
-        error);
-  } else {
-    GST_DEBUG ("requested not to fork for registry rebuild");
-    ret = ensure_current_registry_nonforking (default_registry, registry_file,
-        error);
+  if (do_update) {
+    /* first see if forking is enabled */
+    do_fork = _gst_enable_registry_fork;
+    if (do_fork) {
+      const gchar *fork_env;
+
+      /* forking enabled, see if it is disabled with an env var */
+      if ((fork_env = g_getenv ("GST_REGISTRY_FORK"))) {
+        /* fork enabled for any value different from "no" */
+        do_fork = strcmp (fork_env, "no") != 0;
+      }
+    }
+
+    /* now check registry with or without forking */
+    if (do_fork) {
+      GST_DEBUG ("forking for registry rebuild");
+      ret = ensure_current_registry_forking (default_registry, registry_file,
+          error);
+    } else {
+      GST_DEBUG ("requested not to fork for registry rebuild");
+      ret = ensure_current_registry_nonforking (default_registry, registry_file,
+          error);
+    }
   }
 
   g_free (registry_file);
+  GST_INFO ("registry reading and updating done, result = %d", ret);
 
   return ret;
 }
@@ -932,33 +979,96 @@ init_post (GOptionContext * context, GOptionGroup * group, gpointer data,
   _priv_gst_quarks_initialize ();
   _gst_format_initialize ();
   _gst_query_initialize ();
-  gst_object_get_type ();
-  gst_pad_get_type ();
-  gst_element_factory_get_type ();
-  gst_element_get_type ();
-  gst_type_find_factory_get_type ();
-  gst_bin_get_type ();
+  g_type_class_ref (gst_object_get_type ());
+  g_type_class_ref (gst_pad_get_type ());
+  g_type_class_ref (gst_element_factory_get_type ());
+  g_type_class_ref (gst_element_get_type ());
+  g_type_class_ref (gst_type_find_factory_get_type ());
+  g_type_class_ref (gst_bin_get_type ());
+  g_type_class_ref (gst_bus_get_type ());
+  g_type_class_ref (gst_task_get_type ());
+  g_type_class_ref (gst_clock_get_type ());
 
-#ifndef GST_DISABLE_INDEX
-  gst_index_factory_get_type ();
-#endif /* GST_DISABLE_INDEX */
-#ifndef GST_DISABLE_URI
+  g_type_class_ref (gst_index_factory_get_type ());
   gst_uri_handler_get_type ();
-#endif /* GST_DISABLE_URI */
+
+  g_type_class_ref (gst_object_flags_get_type ());
+  g_type_class_ref (gst_bin_flags_get_type ());
+  g_type_class_ref (gst_buffer_flag_get_type ());
+  g_type_class_ref (gst_buffer_copy_flags_get_type ());
+  g_type_class_ref (gst_bus_flags_get_type ());
+  g_type_class_ref (gst_bus_sync_reply_get_type ());
+  g_type_class_ref (gst_caps_flags_get_type ());
+  g_type_class_ref (gst_clock_return_get_type ());
+  g_type_class_ref (gst_clock_entry_type_get_type ());
+  g_type_class_ref (gst_clock_flags_get_type ());
+  g_type_class_ref (gst_debug_graph_details_get_type ());
+  g_type_class_ref (gst_state_get_type ());
+  g_type_class_ref (gst_state_change_return_get_type ());
+  g_type_class_ref (gst_state_change_get_type ());
+  g_type_class_ref (gst_element_flags_get_type ());
+  g_type_class_ref (gst_core_error_get_type ());
+  g_type_class_ref (gst_library_error_get_type ());
+  g_type_class_ref (gst_resource_error_get_type ());
+  g_type_class_ref (gst_stream_error_get_type ());
+  g_type_class_ref (gst_event_type_flags_get_type ());
+  g_type_class_ref (gst_event_type_get_type ());
+  g_type_class_ref (gst_seek_type_get_type ());
+  g_type_class_ref (gst_seek_flags_get_type ());
+  g_type_class_ref (gst_format_get_type ());
+  g_type_class_ref (gst_index_certainty_get_type ());
+  g_type_class_ref (gst_index_entry_type_get_type ());
+  g_type_class_ref (gst_index_lookup_method_get_type ());
+  g_type_class_ref (gst_assoc_flags_get_type ());
+  g_type_class_ref (gst_index_resolver_method_get_type ());
+  g_type_class_ref (gst_index_flags_get_type ());
+  g_type_class_ref (gst_debug_level_get_type ());
+  g_type_class_ref (gst_debug_color_flags_get_type ());
+  g_type_class_ref (gst_iterator_result_get_type ());
+  g_type_class_ref (gst_iterator_item_get_type ());
+  g_type_class_ref (gst_message_type_get_type ());
+  g_type_class_ref (gst_mini_object_flags_get_type ());
+  g_type_class_ref (gst_pad_link_return_get_type ());
+  g_type_class_ref (gst_flow_return_get_type ());
+  g_type_class_ref (gst_activate_mode_get_type ());
+  g_type_class_ref (gst_pad_direction_get_type ());
+  g_type_class_ref (gst_pad_flags_get_type ());
+  g_type_class_ref (gst_pad_presence_get_type ());
+  g_type_class_ref (gst_pad_template_flags_get_type ());
+  g_type_class_ref (gst_pipeline_flags_get_type ());
+  g_type_class_ref (gst_plugin_error_get_type ());
+  g_type_class_ref (gst_plugin_flags_get_type ());
+  g_type_class_ref (gst_rank_get_type ());
+  g_type_class_ref (gst_query_type_get_type ());
+  g_type_class_ref (gst_buffering_mode_get_type ());
+  g_type_class_ref (gst_tag_merge_mode_get_type ());
+  g_type_class_ref (gst_tag_flag_get_type ());
+  g_type_class_ref (gst_task_state_get_type ());
+  g_type_class_ref (gst_alloc_trace_flags_get_type ());
+  g_type_class_ref (gst_type_find_probability_get_type ());
+  g_type_class_ref (gst_uri_type_get_type ());
+  g_type_class_ref (gst_parse_error_get_type ());
+  g_type_class_ref (gst_parse_flags_get_type ());
 
   gst_structure_get_type ();
   _gst_value_initialize ();
+  g_type_class_ref (gst_param_spec_fraction_get_type ());
   gst_caps_get_type ();
   _gst_event_initialize ();
   _gst_buffer_initialize ();
   _gst_message_initialize ();
   _gst_tag_initialize ();
 
-  /* register core plugins */
-  _gst_plugin_register_static (&plugin_desc);
-
   _gst_plugin_initialize ();
 
+  gst_g_error_get_type ();
+
+  /* register core plugins */
+  gst_plugin_register_static (GST_VERSION_MAJOR, GST_VERSION_MINOR,
+      "staticelements", "core elements linked into the GStreamer library",
+      gst_register_core_elements, VERSION, GST_LICENSE, PACKAGE,
+      GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
+
   /*
    * Any errors happening below this point are non-fatal, we therefore mark
    * gstreamer as being initialized, since it is the case from a plugin point of
@@ -1058,6 +1168,7 @@ gst_debug_help (void)
 }
 #endif
 
+#ifndef GST_DISABLE_OPTION_PARSING
 static gboolean
 parse_one_option (gint opt, const gchar * arg, GError ** err)
 {
@@ -1109,6 +1220,9 @@ parse_one_option (gint opt, const gchar * arg, GError ** err)
     case ARG_SEGTRAP_DISABLE:
       _gst_disable_segtrap = TRUE;
       break;
+    case ARG_REGISTRY_UPDATE_DISABLE:
+      _gst_disable_registry_update = TRUE;
+      break;
     case ARG_REGISTRY_FORK_DISABLE:
       _gst_enable_registry_fork = FALSE;
       break;
@@ -1146,6 +1260,7 @@ parse_goption_arg (const gchar * opt,
     "--gst-plugin-path", ARG_PLUGIN_PATH}, {
     "--gst-plugin-load", ARG_PLUGIN_LOAD}, {
     "--gst-disable-segtrap", ARG_SEGTRAP_DISABLE}, {
+    "--gst-disable-registry-update", ARG_REGISTRY_UPDATE_DISABLE}, {
     "--gst-disable-registry-fork", ARG_REGISTRY_FORK_DISABLE}, {
     NULL}
   };
@@ -1160,6 +1275,7 @@ parse_goption_arg (const gchar * opt,
 
   return parse_one_option (val, arg, err);
 }
+#endif
 
 extern GstRegistry *_gst_registry_default;
 
@@ -1182,7 +1298,7 @@ gst_deinit (void)
 
   GST_INFO ("deinitializing GStreamer");
 
-  if (!gst_initialized) {
+  if (gst_deinitialized) {
     GST_DEBUG ("already deinitialized");
     return;
   }
@@ -1191,9 +1307,11 @@ gst_deinit (void)
   g_slist_free (preload_plugins);
   preload_plugins = NULL;
 
+#ifndef GST_DISABLE_REGISTRY
   g_list_foreach (plugin_paths, (GFunc) g_free, NULL);
   g_list_free (plugin_paths);
   plugin_paths = NULL;
+#endif
 
   clock = gst_system_clock_obtain ();
   gst_object_unref (clock);
@@ -1201,7 +1319,76 @@ gst_deinit (void)
 
   _priv_gst_registry_cleanup ();
 
-  gst_initialized = FALSE;
+  g_type_class_unref (g_type_class_peek (gst_object_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_pad_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_element_factory_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_element_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_type_find_factory_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_bin_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_bus_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_task_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_index_factory_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_object_flags_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_bin_flags_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_buffer_flag_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_buffer_copy_flags_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_bus_flags_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_bus_sync_reply_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_caps_flags_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_clock_return_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_clock_entry_type_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_clock_flags_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_debug_graph_details_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_state_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_state_change_return_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_state_change_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_element_flags_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_core_error_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_library_error_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_resource_error_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_stream_error_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_event_type_flags_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_event_type_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_seek_type_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_seek_flags_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_format_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_index_certainty_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_index_entry_type_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_index_lookup_method_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_assoc_flags_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_index_resolver_method_get_type
+          ()));
+  g_type_class_unref (g_type_class_peek (gst_index_flags_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_debug_level_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_debug_color_flags_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_iterator_result_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_iterator_item_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_message_type_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_mini_object_flags_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_pad_link_return_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_flow_return_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_activate_mode_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_pad_direction_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_pad_flags_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_pad_presence_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_pad_template_flags_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_pipeline_flags_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_plugin_error_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_plugin_flags_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_rank_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_query_type_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_buffering_mode_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_tag_merge_mode_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_tag_flag_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_task_state_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_alloc_trace_flags_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_type_find_probability_get_type
+          ()));
+  g_type_class_unref (g_type_class_peek (gst_uri_type_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_parse_error_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_param_spec_fraction_get_type ()));
+
+  gst_deinitialized = TRUE;
   GST_INFO ("deinitialized GStreamer");
 }
 
