@@ -56,6 +56,8 @@
 #include "config.h"
 #endif
 
+#include "gst_private.h"
+
 #include <sys/types.h>
 
 #ifdef HAVE_UNISTD_H
@@ -80,18 +82,20 @@
 /* OS/X needs this because of bad headers */
 #include <string.h>
 
-#include "gst_private.h"
-
 #include "gstpoll.h"
 
 #ifndef G_OS_WIN32
 /* the poll/select call is also performed on a control socket, that way
  * we can send special commands to control it
  */
+/* FIXME: Shouldn't we check or return the return value
+ * of write()?
+ */
 #define SEND_COMMAND(set, command)                   \
 G_STMT_START {                                       \
   unsigned char c = command;                         \
-  write (set->control_write_fd.fd, &c, 1);           \
+  ssize_t res;                                       \
+  res = write (set->control_write_fd.fd, &c, 1);     \
 } G_STMT_END
 
 #define READ_COMMAND(set, command, res)              \
