@@ -46,6 +46,8 @@ G_BEGIN_DECLS
  * @GST_QUERY_SEGMENT: segment start/stop positions
  * @GST_QUERY_CONVERT: convert values between formats
  * @GST_QUERY_FORMATS: query supported formats for convert
+ * @GST_QUERY_BUFFERING: query available media for efficient seeking. Since
+ * 0.10.20.
  *
  * Standard predefined Query types
  */
@@ -61,9 +63,26 @@ typedef enum {
   GST_QUERY_SEEKING,
   GST_QUERY_SEGMENT,
   GST_QUERY_CONVERT,
-  GST_QUERY_FORMATS
+  GST_QUERY_FORMATS,
+  GST_QUERY_BUFFERING
 } GstQueryType;
 
+/**
+ * GstBufferingMode:
+ * @GST_BUFFERING_STREAM: a small amount of data is buffered
+ * @GST_BUFFERING_DOWNLOAD: the stream is being downloaded
+ * @GST_BUFFERING_TIMESHIFT: the stream is being downloaded in a ringbuffer
+ * @GST_BUFFERING_LIVE: the stream is a live stream
+ *
+ * The different types of buffering methods.
+ */
+typedef enum {
+  GST_BUFFERING_STREAM,
+  GST_BUFFERING_DOWNLOAD,
+  GST_BUFFERING_TIMESHIFT,
+  GST_BUFFERING_LIVE
+} GstBufferingMode;
+
 typedef struct _GstQueryTypeDefinition GstQueryTypeDefinition;
 typedef struct _GstQuery GstQuery;
 typedef struct _GstQueryClass GstQueryClass;
@@ -80,8 +99,8 @@ typedef struct _GstQueryClass GstQueryClass;
 struct _GstQueryTypeDefinition
 {
   GstQueryType   value;
-  gchar     	*nick;
-  gchar     	*description;
+  gchar  	*nick;
+  gchar  	*description;
   GQuark         quark;
 };
 
@@ -139,8 +158,6 @@ struct _GstQueryClass {
   gpointer _gst_reserved[GST_PADDING];
 };
 
-void            _gst_query_initialize          (void);
-
 const gchar*    gst_query_type_get_name        (GstQueryType query);
 GQuark          gst_query_type_to_quark        (GstQueryType query);
 
@@ -205,6 +222,13 @@ GstQuery*	gst_query_new_duration		(GstFormat format);
 void		gst_query_set_duration		(GstQuery *query, GstFormat format, gint64 duration);
 void		gst_query_parse_duration	(GstQuery *query, GstFormat *format, gint64 *duration);
 
+/* latency query */
+GstQuery*	gst_query_new_latency		(void);
+void		gst_query_set_latency		(GstQuery *query, gboolean live, GstClockTime min_latency,
+		                                 GstClockTime max_latency);
+void		gst_query_parse_latency		(GstQuery *query, gboolean *live, GstClockTime *min_latency, 
+		                                 GstClockTime *max_latency);
+
 /* convert query */
 GstQuery*	gst_query_new_convert		(GstFormat src_format, gint64 value, GstFormat dest_format);
 void		gst_query_set_convert		(GstQuery *query, GstFormat src_format, gint64 src_value,
@@ -218,15 +242,12 @@ void            gst_query_set_segment           (GstQuery *query, gdouble rate,
 void            gst_query_parse_segment         (GstQuery *query, gdouble *rate, GstFormat *format,
                                                  gint64 *start_value, gint64 *stop_value);
 
-
 /* application specific query */
 GstQuery *	gst_query_new_application 	(GstQueryType type,
                                                  GstStructure *structure);
-
-
 GstStructure *  gst_query_get_structure		(GstQuery *query);
 
-/* moved from old gstqueryutils.h */
+/* seeking query */
 GstQuery*       gst_query_new_seeking           (GstFormat format);
 void            gst_query_set_seeking           (GstQuery *query, GstFormat format,
                                                  gboolean seekable,
@@ -236,13 +257,32 @@ void            gst_query_parse_seeking         (GstQuery *query, GstFormat *for
                                                  gboolean *seekable,
                                                  gint64 *segment_start,
                                                  gint64 *segment_end);
-
+/* formats query */
 GstQuery*       gst_query_new_formats           (void);
 void            gst_query_set_formats           (GstQuery *query, gint n_formats, ...);
 void            gst_query_set_formatsv          (GstQuery *query, gint n_formats, GstFormat *formats);
 void            gst_query_parse_formats_length  (GstQuery *query, guint *n_formats);
 void            gst_query_parse_formats_nth     (GstQuery *query, guint nth, GstFormat *format);
 
+/* buffering query */
+GstQuery*       gst_query_new_buffering           (GstFormat format);
+void            gst_query_set_buffering_percent   (GstQuery *query, gboolean busy, gint percent);
+void            gst_query_parse_buffering_percent (GstQuery *query, gboolean *busy, gint *percent);
+
+void            gst_query_set_buffering_stats     (GstQuery *query, GstBufferingMode mode,
+                                                   gint avg_in, gint avg_out, 
+						   gint64 buffering_left); 
+void            gst_query_parse_buffering_stats    (GstQuery *query, GstBufferingMode *mode,
+                                                   gint *avg_in, gint *avg_out, 
+						   gint64 *buffering_left); 
+
+void            gst_query_set_buffering_range     (GstQuery *query, GstFormat format,
+                                                   gint64 start, gint64 stop,
+						   gint64 estimated_total);
+void            gst_query_parse_buffering_range   (GstQuery *query, GstFormat *format,
+                                                   gint64 *start, gint64 *stop,
+						   gint64 *estimated_total);
+
 G_END_DECLS
 
 #endif /* __GST_QUERY_H__ */
