@@ -113,7 +113,7 @@ gst_structure_id_empty_new_with_size (GQuark quark, guint prealloc)
 {
   GstStructure *structure;
 
-  structure = g_new0 (GstStructure, 1);
+  structure = g_slice_new0 (GstStructure);
   structure->type = gst_structure_get_type ();
   structure->name = quark;
   structure->fields =
@@ -145,7 +145,8 @@ gst_structure_validate_name (const gchar * name)
 
   g_return_val_if_fail (name != NULL, FALSE);
 
-  if (!g_ascii_isalpha (*name)) {
+  /* FIXME 0.11: use g_ascii_isalpha() */
+  if (!g_ascii_isalnum (*name)) {
     GST_WARNING ("Invalid character '%c' at offset 0 in structure name: %s",
         *name, name);
     return FALSE;
@@ -236,7 +237,9 @@ gst_structure_new_valist (const gchar * name,
   g_return_val_if_fail (name != NULL, NULL);
 
   structure = gst_structure_empty_new (name);
-  gst_structure_set_valist (structure, firstfield, varargs);
+
+  if (structure)
+    gst_structure_set_valist (structure, firstfield, varargs);
 
   return structure;
 }
@@ -327,7 +330,7 @@ gst_structure_free (GstStructure * structure)
 #ifdef USE_POISONING
   memset (structure, 0xff, sizeof (GstStructure));
 #endif
-  g_free (structure);
+  g_slice_free (GstStructure, structure);
 }
 
 /**
@@ -426,20 +429,16 @@ gst_structure_id_set_value (GstStructure * structure,
   g_return_if_fail (G_IS_VALUE (value));
   g_return_if_fail (IS_MUTABLE (structure));
 
-  /* if someones disables GST_DEBUG output, they probably do it for
-   * performance reasons, so skip the UTF-8 check here as well then */
-#ifndef GST_DISABLE_GST_DEBUG
   if (G_VALUE_HOLDS_STRING (value)) {
     const gchar *s;
 
     s = g_value_get_string (value);
-    if (s != NULL && !g_utf8_validate (s, -1, NULL)) {
+    if (G_UNLIKELY (s != NULL && !g_utf8_validate (s, -1, NULL))) {
       g_warning ("Trying to set string field '%s' on structure, but string is "
           "not valid UTF-8. Please file a bug.", g_quark_to_string (field));
       return;
     }
   }
-#endif
 
   gsfield.name = field;
   gst_value_init_and_copy (&gsfield.value, value);
@@ -1507,6 +1506,39 @@ gst_structure_value_get_generic_type (GValue * val)
       ((c) == '-') || ((c) == '+') || ((c) == '/') || ((c) == ':') || \
       ((c) == '.'))
 
+gboolean
+priv_gst_structure_append_to_gstring (const GstStructure * structure,
+    GString * s)
+{
+  GstStructureField *field;
+  guint i;
+
+  g_return_val_if_fail (s != NULL, FALSE);
+
+  g_string_append (s, g_quark_to_string (structure->name));
+  for (i = 0; i < structure->fields->len; i++) {
+    char *t;
+    GType type;
+
+    field = GST_STRUCTURE_FIELD (structure, i);
+
+    t = gst_value_serialize (&field->value);
+    type = gst_structure_value_get_generic_type (&field->value);
+
+    g_string_append_len (s, ", ", 2);
+    /* FIXME: do we need to escape fieldnames? */
+    g_string_append (s, g_quark_to_string (field->name));
+    g_string_append_len (s, "=(", 2);
+    g_string_append (s, gst_structure_to_abbr (type));
+    g_string_append_c (s, ')');
+    g_string_append (s, GST_STR_NULL (t));
+    g_free (t);
+  }
+
+  g_string_append_c (s, ';');
+  return TRUE;
+}
+
 /**
  * gst_structure_to_string:
  * @structure: a #GstStructure
@@ -1519,9 +1551,7 @@ gst_structure_value_get_generic_type (GValue * val)
 gchar *
 gst_structure_to_string (const GstStructure * structure)
 {
-  GstStructureField *field;
   GString *s;
-  guint i;
 
   /* NOTE:  This function is potentially called by the debug system,
    * so any calls to gst_log() (and GST_DEBUG(), GST_LOG(), etc.)
@@ -1531,25 +1561,10 @@ gst_structure_to_string (const GstStructure * structure)
 
   g_return_val_if_fail (structure != NULL, NULL);
 
-  s = g_string_new ("");
-  g_string_append_printf (s, "%s", g_quark_to_string (structure->name));
-  for (i = 0; i < structure->fields->len; i++) {
-    char *t;
-    GType type;
-
-    field = GST_STRUCTURE_FIELD (structure, i);
-
-    t = gst_value_serialize (&field->value);
-    type = gst_structure_value_get_generic_type (&field->value);
-
-    /* FIXME: do we need to escape fieldnames? */
-    g_string_append_printf (s, ", %s=(%s)%s", g_quark_to_string (field->name),
-        gst_structure_to_abbr (type), GST_STR_NULL (t));
-    g_free (t);
-  }
-
-  g_string_append_c (s, ';');
-
+  /* we estimate a minimum size based on the number of fields in order to
+   * avoid unnecessary reallocs within GString */
+  s = g_string_sized_new (STRUCTURE_ESTIMATED_STRING_LEN (structure));
+  priv_gst_structure_append_to_gstring (structure, s);
   return g_string_free (s, FALSE);
 }
 
@@ -1883,7 +1898,7 @@ gst_structure_parse_value (gchar * str,
  * where parsing ended will be returned.
  *
  * Returns: a new #GstStructure or NULL when the string could not
- * be parsed. Free after usage.
+ * be parsed. Free with gst_structure_free() after use.
  */
 GstStructure *
 gst_structure_from_string (const gchar * string, gchar ** end)
@@ -1901,7 +1916,7 @@ gst_structure_from_string (const gchar * string, gchar ** end)
   copy = g_strdup (string);
   r = copy;
 
-  /* skipe spaces */
+  /* skip spaces (FIXME: _isspace treats tabs and newlines as space!) */
   while (*r && (g_ascii_isspace (*r) || (r[0] == '\\'
               && g_ascii_isspace (r[1]))))
     r++;
@@ -1917,6 +1932,9 @@ gst_structure_from_string (const gchar * string, gchar ** end)
   structure = gst_structure_empty_new (name);
   *w = save;
 
+  if (structure == NULL)
+    goto error;
+
   do {
     while (*r && (g_ascii_isspace (*r) || (r[0] == '\\'
                 && g_ascii_isspace (r[1]))))
@@ -1948,6 +1966,9 @@ gst_structure_from_string (const gchar * string, gchar ** end)
 
   if (end)
     *end = (char *) string + (r - copy);
+  else if (*r)
+    g_warning ("gst_structure_from_string did not consume whole string,"
+        " but caller did not provide end pointer (\"%s\")", string);
 
   g_free (copy);
   return structure;
@@ -2211,16 +2232,13 @@ gst_structure_fixate_field_nearest_fraction (GstStructure * structure,
     const GValue *list_value;
     int i, n;
     const GValue *best = NULL;
-    GValue best_diff = { 0 };
-    GValue cur_diff = { 0 };
-    GValue target = { 0 };
-    gboolean res = FALSE;
+    gdouble target;
+    gdouble cur_diff;
+    gdouble best_diff = G_MAXDOUBLE;
 
-    g_value_init (&best_diff, GST_TYPE_FRACTION);
-    g_value_init (&cur_diff, GST_TYPE_FRACTION);
-    g_value_init (&target, GST_TYPE_FRACTION);
+    target = (gdouble) target_numerator / (gdouble) target_denominator;
 
-    gst_value_set_fraction (&target, target_numerator, target_denominator);
+    GST_DEBUG ("target %g, best %g", target, best_diff);
 
     best = NULL;
 
@@ -2228,28 +2246,32 @@ gst_structure_fixate_field_nearest_fraction (GstStructure * structure,
     for (i = 0; i < n; i++) {
       list_value = gst_value_list_get_value (value, i);
       if (G_VALUE_TYPE (list_value) == GST_TYPE_FRACTION) {
+        gint num, denom;
+        gdouble list_double;
 
-        if (gst_value_compare (list_value, &target) == GST_VALUE_LESS_THAN)
-          gst_value_fraction_subtract (&cur_diff, &target, list_value);
-        else
-          gst_value_fraction_subtract (&cur_diff, list_value, &target);
+        num = gst_value_get_fraction_numerator (list_value);
+        denom = gst_value_get_fraction_denominator (list_value);
 
-        if (!best
-            || gst_value_compare (&cur_diff,
-                &best_diff) == GST_VALUE_LESS_THAN) {
+        list_double = ((gdouble) num / (gdouble) denom);
+        cur_diff = target - list_double;
+
+        GST_DEBUG ("curr diff %g, list %g", cur_diff, list_double);
+
+        if (cur_diff < 0)
+          cur_diff = -cur_diff;
+
+        if (!best || cur_diff < best_diff) {
+          GST_DEBUG ("new best %g", list_double);
           best = list_value;
-          g_value_copy (&cur_diff, &best_diff);
+          best_diff = cur_diff;
         }
       }
     }
     if (best != NULL) {
       gst_structure_set_value (structure, field_name, best);
-      res = TRUE;
+      return TRUE;
     }
-    g_value_unset (&best_diff);
-    g_value_unset (&cur_diff);
-    g_value_unset (&target);
-    return res;
   }
+
   return FALSE;
 }
