@@ -65,6 +65,7 @@ static GstClockReturn gst_system_clock_id_wait_async (GstClock * clock,
 static void gst_system_clock_id_unschedule (GstClock * clock,
     GstClockEntry * entry);
 static void gst_system_clock_async_thread (GstClock * clock);
+static gboolean gst_system_clock_start_async (GstSystemClock * clock);
 
 static GStaticMutex _gst_sysclock_mutex = G_STATIC_MUTEX_INIT;
 
@@ -122,31 +123,18 @@ gst_system_clock_class_init (GstSystemClockClass * klass)
 static void
 gst_system_clock_init (GstSystemClock * clock)
 {
-  GError *error = NULL;
-
   GST_OBJECT_FLAG_SET (clock,
       GST_CLOCK_FLAG_CAN_DO_SINGLE_SYNC |
       GST_CLOCK_FLAG_CAN_DO_SINGLE_ASYNC |
       GST_CLOCK_FLAG_CAN_DO_PERIODIC_SYNC |
       GST_CLOCK_FLAG_CAN_DO_PERIODIC_ASYNC);
 
-  GST_OBJECT_LOCK (clock);
-  clock->thread = g_thread_create ((GThreadFunc) gst_system_clock_async_thread,
-      clock, TRUE, &error);
-  if (error)
-    goto no_thread;
-
-  /* wait for it to spin up */
-  GST_CLOCK_WAIT (clock);
-  GST_OBJECT_UNLOCK (clock);
-  return;
-
-  /* ERRORS */
-no_thread:
-  {
-    g_warning ("could not create async clock thread: %s", error->message);
-    GST_OBJECT_UNLOCK (clock);
-  }
+#if 0
+  /* Uncomment this to start the async clock thread straight away */
+  GST_CLOCK_LOCK (clock);
+  gst_system_clock_start_async (clock);
+  GST_CLOCK_UNLOCK (clock);
+#endif
 }
 
 static void
@@ -375,7 +363,8 @@ gst_system_clock_id_wait_jitter_unlocked (GstClock * clock,
   GstClockTime entryt, real, now, target;
   GstClockTimeDiff diff;
 
-  /* need to call the overridden method */
+  /* need to call the overridden method because we want to sync against the time
+   * of the clock, whatever the subclass uses as a clock. */
   real = GST_CLOCK_GET_CLASS (clock)->get_internal_time (clock);
   entryt = GST_CLOCK_ENTRY_TIME (entry);
 
@@ -383,7 +372,13 @@ gst_system_clock_id_wait_jitter_unlocked (GstClock * clock,
   if (jitter) {
     *jitter = GST_CLOCK_DIFF (entryt, now);
   }
+  /* the diff of the entry with the clock is the amount of time we have to
+   * wait */
   diff = entryt - now;
+  /* Our GCond implementation expects an absolute time against the system clock
+   * as a timeout value. We use our internal time to get the system time and add
+   * the expected timeout to it, this gives us the absolute time of the
+   * timeout. */
   target = gst_system_clock_get_internal_time (clock) + diff;
 
   GST_CAT_DEBUG (GST_CAT_CLOCK, "entry %p"
@@ -391,7 +386,7 @@ gst_system_clock_id_wait_jitter_unlocked (GstClock * clock,
       " entry %" GST_TIME_FORMAT
       " now %" GST_TIME_FORMAT
       " real %" GST_TIME_FORMAT
-      " diff %" G_GINT64_FORMAT,
+      " diff (entry-now) %" G_GINT64_FORMAT,
       entry,
       GST_TIME_ARGS (target),
       GST_TIME_ARGS (entryt), GST_TIME_ARGS (now), GST_TIME_ARGS (real), diff);
@@ -436,6 +431,10 @@ gst_system_clock_id_wait_jitter_unlocked (GstClock * clock,
         /* else restart if we must */
         if (!restart)
           break;
+
+        /* this can happen if the entry got unlocked because of an async entry
+         * was added to the head of the async queue. */
+        GST_CAT_DEBUG (GST_CAT_CLOCK, "continue waiting for entry %p", entry);
       }
     }
   } else if (diff == 0) {
@@ -453,12 +452,40 @@ gst_system_clock_id_wait_jitter (GstClock * clock, GstClockEntry * entry,
   GstClockReturn ret;
 
   GST_OBJECT_LOCK (clock);
-  ret = gst_system_clock_id_wait_jitter_unlocked (clock, entry, jitter, FALSE);
+  ret = gst_system_clock_id_wait_jitter_unlocked (clock, entry, jitter, TRUE);
   GST_OBJECT_UNLOCK (clock);
 
   return ret;
 }
 
+/* Start the async clock thread. Must be called with the object lock
+ * held */
+static gboolean
+gst_system_clock_start_async (GstSystemClock * clock)
+{
+  GError *error = NULL;
+
+  if (clock->thread != NULL)
+    return TRUE;                /* Thread already running. Nothing to do */
+
+  clock->thread = g_thread_create ((GThreadFunc) gst_system_clock_async_thread,
+      clock, TRUE, &error);
+  if (error)
+    goto no_thread;
+
+  /* wait for it to spin up */
+  GST_CLOCK_WAIT (clock);
+
+  return TRUE;
+
+  /* ERRORS */
+no_thread:
+  {
+    g_warning ("could not create async clock thread: %s", error->message);
+  }
+  return FALSE;
+}
+
 /* Add an entry to the list of pending async waits. The entry is inserted
  * in sorted order. If we inserted the entry at the head of the list, we
  * need to signal the thread as it might either be waiting on it or waiting
@@ -469,9 +496,14 @@ gst_system_clock_id_wait_jitter (GstClock * clock, GstClockEntry * entry,
 static GstClockReturn
 gst_system_clock_id_wait_async (GstClock * clock, GstClockEntry * entry)
 {
-  GST_CAT_DEBUG (GST_CAT_CLOCK, "adding entry %p", entry);
+  GST_CAT_DEBUG (GST_CAT_CLOCK, "adding async entry %p", entry);
 
   GST_OBJECT_LOCK (clock);
+
+  /* Start the clock async thread if needed */
+  if (!gst_system_clock_start_async (GST_SYSTEM_CLOCK (clock)))
+    goto thread_error;
+
   /* need to take a ref */
   gst_clock_id_ref ((GstClockID) entry);
   /* insert the entry in sorted order */
@@ -482,12 +514,21 @@ gst_system_clock_id_wait_async (GstClock * clock, GstClockEntry * entry)
    * front, else the thread is just waiting for another entry and
    * will get to this entry automatically. */
   if (clock->entries->data == entry) {
-    GST_CAT_DEBUG (GST_CAT_CLOCK, "send signal");
+    GST_CAT_DEBUG (GST_CAT_CLOCK, "async entry added to head, sending signal");
+    /* this will wake up _all_ entries waiting for the clock because we have
+     * only one cond for all entries (makes allocation faster). Entries that
+     * have not timed out will have their status set to BUSY and should continue
+     * to wait. In the case of the async ones, the new head entry should be
+     * taken and waited for. */
     GST_CLOCK_BROADCAST (clock);
   }
   GST_OBJECT_UNLOCK (clock);
 
   return GST_CLOCK_OK;
+
+thread_error:
+  /* Could not start the async clock thread */
+  return GST_CLOCK_ERROR;
 }
 
 /* unschedule an entry. This will set the state of the entry to GST_CLOCK_UNSCHEDULED
@@ -503,8 +544,11 @@ gst_system_clock_id_unschedule (GstClock * clock, GstClockEntry * entry)
   GST_CAT_DEBUG (GST_CAT_CLOCK, "unscheduling entry %p", entry);
 
   GST_OBJECT_LOCK (clock);
+  /* mark entry as unscheduled, then wake up all entries. The entries that did
+   * not timeout will be woken up but immediatly go to sleep again because their
+   * status would still be busy. */
   entry->status = GST_CLOCK_UNSCHEDULED;
-  GST_CAT_DEBUG (GST_CAT_CLOCK, "send signal");
+  GST_CAT_DEBUG (GST_CAT_CLOCK, "sending signal");
   GST_CLOCK_BROADCAST (clock);
   GST_OBJECT_UNLOCK (clock);
 }
