@@ -56,6 +56,10 @@
 #include <fcntl.h>
 #include <string.h>
 
+#if defined (_MSC_VER) && _MSC_VER >= 1400
+# include <io.h>
+#endif
+
 #include "gst_private.h"
 #include "gstinfo.h"
 
@@ -91,7 +95,7 @@ gst_trace_read_tsc (gint64 * dst)
   read_tsc (dst);
 }
 
-GstTrace *_gst_trace_default = NULL;
+static GstTrace *_gst_trace_default = NULL;
 gint _gst_trace_on = 1;
 
 /**
