@@ -35,8 +35,8 @@
 #include "gstregistry.h"
 #include "gsttypefindfactory.h"
 
-GST_DEBUG_CATEGORY_EXTERN (gst_type_find_debug);
-#define GST_CAT_DEFAULT gst_type_find_debug
+GST_DEBUG_CATEGORY_EXTERN (type_find_debug);
+#define GST_CAT_DEFAULT type_find_debug
 
 GType
 gst_type_find_get_type (void)
@@ -51,7 +51,8 @@ gst_type_find_get_type (void)
 
 /**
  * gst_type_find_register:
- * @plugin: A #GstPlugin.
+ * @plugin: A #GstPlugin, or NULL for a static typefind function (note that
+ *    passing NULL only works in GStreamer 0.10.16 and later)
  * @name: The name for registering
  * @rank: The rank (or importance) of this typefind function
  * @func: The #GstTypeFindFunction to use
@@ -76,7 +77,6 @@ gst_type_find_register (GstPlugin * plugin, const gchar * name, guint rank,
 {
   GstTypeFindFactory *factory;
 
-  g_return_val_if_fail (plugin != NULL, FALSE);
   g_return_val_if_fail (name != NULL, FALSE);
   g_return_val_if_fail (func != NULL, FALSE);
 
@@ -96,7 +96,11 @@ gst_type_find_register (GstPlugin * plugin, const gchar * name, guint rank,
   factory->function = func;
   factory->user_data = data;
   factory->user_data_notify = data_notify;
-  GST_PLUGIN_FEATURE (factory)->plugin_name = plugin->desc.name;        /* interned string */
+  if (plugin && plugin->desc.name) {
+    GST_PLUGIN_FEATURE (factory)->plugin_name = plugin->desc.name;      /* interned string */
+  } else {
+    GST_PLUGIN_FEATURE (factory)->plugin_name = "NULL";
+  }
   GST_PLUGIN_FEATURE (factory)->loaded = TRUE;
 
   gst_registry_add_feature (gst_registry_get_default (),
@@ -152,6 +156,57 @@ gst_type_find_suggest (GstTypeFind * find, guint probability,
   find->suggest (find->data, probability, caps);
 }
 
+/**
+ * gst_type_find_suggest_simple:
+ * @find: The #GstTypeFind object the function was called with
+ * @probability: The probability in percent that the suggestion is right
+ * @media_type: the media type of the suggested caps
+ * @fieldname: first field of the suggested caps, or NULL
+ * @...: additional arguments to the suggested caps in the same format as the
+ *     arguments passed to gst_structure_new() (ie. triplets of field name,
+ *     field GType and field value)
+ *
+ * If a #GstTypeFindFunction calls this function it suggests the caps with the
+ * given probability. A #GstTypeFindFunction may supply different suggestions
+ * in one call. It is up to the caller of the #GstTypeFindFunction to interpret
+ * these values.
+ *
+ * This function is similar to gst_type_find_suggest(), only that instead of
+ * passing a #GstCaps argument you can create the caps on the fly in the same
+ * way as you can with gst_caps_new_simple().
+ *
+ * Make sure you terminate the list of arguments with a NULL argument and that
+ * the values passed have the correct type (in terms of width in bytes when
+ * passed to the vararg function - this applies particularly to gdouble and
+ * guint64 arguments).
+ *
+ * Since: 0.10.20
+ */
+void
+gst_type_find_suggest_simple (GstTypeFind * find, guint probability,
+    const char *media_type, const char *fieldname, ...)
+{
+  GstStructure *structure;
+  va_list var_args;
+  GstCaps *caps;
+
+  g_return_if_fail (find->suggest != NULL);
+  g_return_if_fail (probability <= 100);
+  g_return_if_fail (media_type != NULL);
+
+  caps = gst_caps_new_empty ();
+
+  va_start (var_args, fieldname);
+  structure = gst_structure_new_valist (media_type, fieldname, var_args);
+  va_end (var_args);
+
+  gst_caps_append_structure (caps, structure);
+  g_return_if_fail (gst_caps_is_fixed (caps));
+
+  find->suggest (find->data, probability, caps);
+  gst_caps_unref (caps);
+}
+
 /**
  * gst_type_find_get_length:
  * @find: The #GstTypeFind the function was called with
