@@ -3,6 +3,7 @@
  * Common code for GStreamer unittests
  *
  * Copyright (C) <2004> Thomas Vander Stichele <thomas at apestaart dot org>
+ * Copyright (C) <2008> Thijs Vermeir <thijsvermeir@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -48,10 +49,10 @@ extern gboolean _gst_check_raised_warning;
 extern gboolean _gst_check_expecting_log;
 
 /* global variables used in test methods */
-GList * buffers;
+extern GList * buffers;
 
-GMutex *check_mutex;
-GCond *check_cond;
+extern GMutex *check_mutex;
+extern GCond *check_cond;
 
 typedef struct
 {
@@ -63,30 +64,53 @@ GstCheckABIStruct;
 
 void gst_check_init (int *argc, char **argv[]);
 
-GstFlowReturn gst_check_chain_func (GstPad *pad, GstBuffer *buffer);
+GstFlowReturn gst_check_chain_func (GstPad * pad, GstBuffer * buffer);
 
-void gst_check_message_error (GstMessage *message, GstMessageType type, GQuark domain, gint code);
+void gst_check_message_error (GstMessage * message, GstMessageType type,
+    GQuark domain, gint code);
 
-GstElement * gst_check_setup_element (const gchar *factory);
-void gst_check_teardown_element (GstElement *element);
-GstPad * gst_check_setup_src_pad (GstElement *element,
-    GstStaticPadTemplate *template, GstCaps *caps);
-void gst_check_teardown_src_pad (GstElement *element);
-GstPad * gst_check_setup_sink_pad (GstElement *element,
-    GstStaticPadTemplate *template, GstCaps *caps);
-void gst_check_teardown_sink_pad (GstElement *element);
+GstElement *gst_check_setup_element (const gchar * factory);
+void gst_check_teardown_element (GstElement * element);
+GstPad *gst_check_setup_src_pad (GstElement * element,
+    GstStaticPadTemplate * template, GstCaps * caps);
+GstPad * gst_check_setup_src_pad_by_name (GstElement * element,
+          GstStaticPadTemplate * template, gchar *name);
+GstPad * gst_check_setup_sink_pad_by_name (GstElement * element, 
+          GstStaticPadTemplate * template, gchar *name);
+void gst_check_teardown_pad_by_name (GstElement * element, gchar *name);
+void gst_check_teardown_src_pad (GstElement * element);
+void gst_check_drop_buffers ();
+void gst_check_caps_equal (GstCaps * caps1, GstCaps * caps2);
+void gst_check_element_push_buffer_list (const gchar * element_name,
+    GList * buffer_in, GList * buffer_out, GstFlowReturn last_flow_return);
+void gst_check_element_push_buffer (const gchar * element_name,
+    GstBuffer * buffer_in, GstBuffer * buffer_out);
+GstPad *gst_check_setup_sink_pad (GstElement * element,
+    GstStaticPadTemplate * template, GstCaps * caps);
+void gst_check_teardown_sink_pad (GstElement * element);
 void gst_check_abi_list (GstCheckABIStruct list[], gboolean have_abi_sizes);
-gint gst_check_run_suite (Suite *suite, const gchar *name, const gchar *fname);
+gint gst_check_run_suite (Suite * suite, const gchar * name,
+    const gchar * fname);
 
 #define fail_unless_message_error(msg, domain, code)		\
 gst_check_message_error (msg, GST_MESSAGE_ERROR,		\
   GST_ ## domain ## _ERROR, GST_ ## domain ## _ERROR_ ## code)
 #define assert_message_error(m, d, c) fail_unless_message_error(m, d, c)
 
-/***
- * wrappers for START_TEST and END_TEST
+/**
+ * GST_START_TEST:
+ * @__testname: test function name
+ *
+ * wrapper for checks START_TEST
  */
-#if CHECK_MAJOR_VERSION >= 0 && CHECK_MINOR_VERSION >= 9 && CHECK_MICRO_VERSION >= 4
+/**
+ * GST_END_TEST:
+ *
+ * wrapper for checks END_TEST
+ */
+#if CHECK_MAJOR_VERSION > 0 || \
+    (CHECK_MAJOR_VERSION == 0 && CHECK_MINOR_VERSION > 9) || \
+    (CHECK_MAJOR_VERSION == 0 && CHECK_MINOR_VERSION == 9 && CHECK_MICRO_VERSION > 3)
 #define GST_START_TEST(__testname) \
 static void __testname (int __i__)\
 {\
@@ -388,6 +412,34 @@ int main (int argc, char **argv)				\
   return gst_check_run_suite (s, # name, __FILE__);		\
 }
 
+/* Hack to allow run-time selection of unit tests to run via the
+ * GST_CHECKS environment variable (test function names, comma-separated) */
+
+gboolean _gst_check_run_test_func (const gchar * func_name);
+
+#if CHECK_MAJOR_VERSION > 0 || \
+    (CHECK_MAJOR_VERSION == 0 && CHECK_MINOR_VERSION > 9) || \
+    (CHECK_MAJOR_VERSION == 0 && CHECK_MINOR_VERSION == 9 && CHECK_MICRO_VERSION > 3)
+static inline void
+__gst_tcase_add_test (TCase * tc, TFun tf, const char * fname, int signal,
+    int start, int end)
+{
+  if (_gst_check_run_test_func (fname)) {
+    _tcase_add_test (tc, tf, fname, signal, start, end);
+  }
+}
+#else
+static inline void
+__gst_tcase_add_test (TCase * tc, TFun tf, const char * fname, int signal)
+{
+  if (_gst_check_run_test_func (fname)) {
+    _tcase_add_test (tc, tf, fname, signal);
+  }
+}
+#endif
+
+#define _tcase_add_test __gst_tcase_add_test
+
 G_END_DECLS
 
 #endif /* __GST_CHECK_H__ */
