@@ -38,11 +38,9 @@
 #include "gstcontrollerprivate.h"
 #include "gstcontroller.h"
 
-#define GST_CAT_DEFAULT gst_controller_debug
+#define GST_CAT_DEFAULT controller_debug
 GST_DEBUG_CATEGORY_EXTERN (GST_CAT_DEFAULT);
 
-extern GQuark __gst_controller_key;
-
 /**
  * gst_object_control_properties:
  * @object: the object of which some properties should be controlled
@@ -64,7 +62,7 @@ gst_object_control_properties (GObject * object, ...)
   GstController *ctrl;
   va_list var_args;
 
-  g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
+  g_return_val_if_fail (G_IS_OBJECT (object), NULL);
 
   va_start (var_args, object);
   ctrl = gst_controller_new_valist (object, var_args);
@@ -93,7 +91,7 @@ gst_object_uncontrol_properties (GObject * object, ...)
 
   g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
 
-  if ((ctrl = g_object_get_qdata (object, __gst_controller_key))) {
+  if ((ctrl = g_object_get_qdata (object, priv_gst_controller_key))) {
     va_list var_args;
 
     va_start (var_args, object);
@@ -115,9 +113,9 @@ gst_object_uncontrol_properties (GObject * object, ...)
 GstController *
 gst_object_get_controller (GObject * object)
 {
-  g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
+  g_return_val_if_fail (G_IS_OBJECT (object), NULL);
 
-  return (g_object_get_qdata (object, __gst_controller_key));
+  return (g_object_get_qdata (object, priv_gst_controller_key));
 }
 
 /**
@@ -138,8 +136,8 @@ gst_object_set_controller (GObject * object, GstController * controller)
   g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
   g_return_val_if_fail (controller, FALSE);
 
-  if (!(ctrl = g_object_get_qdata (object, __gst_controller_key))) {
-    g_object_set_qdata (object, __gst_controller_key, controller);
+  if (!(ctrl = g_object_get_qdata (object, priv_gst_controller_key))) {
+    g_object_set_qdata (object, priv_gst_controller_key, controller);
     return (TRUE);
   }
   return (FALSE);
@@ -161,7 +159,7 @@ gst_object_suggest_next_sync (GObject * object)
 
   g_return_val_if_fail (G_IS_OBJECT (object), GST_CLOCK_TIME_NONE);
 
-  if ((ctrl = g_object_get_qdata (object, __gst_controller_key))) {
+  if ((ctrl = g_object_get_qdata (object, priv_gst_controller_key))) {
     return gst_controller_suggest_next_sync (ctrl);
   }
   return (GST_CLOCK_TIME_NONE);
@@ -185,7 +183,7 @@ gst_object_sync_values (GObject * object, GstClockTime timestamp)
   g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
   g_return_val_if_fail (GST_CLOCK_TIME_IS_VALID (timestamp), FALSE);
 
-  if ((ctrl = g_object_get_qdata (object, __gst_controller_key))) {
+  if ((ctrl = g_object_get_qdata (object, priv_gst_controller_key))) {
     return gst_controller_sync_values (ctrl, timestamp);
   }
   return (FALSE);
@@ -214,7 +212,7 @@ gst_object_set_control_source (GObject * object, gchar * property_name,
   g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
   g_return_val_if_fail (GST_IS_CONTROL_SOURCE (csource), FALSE);
 
-  if ((ctrl = g_object_get_qdata (object, __gst_controller_key))) {
+  if ((ctrl = g_object_get_qdata (object, priv_gst_controller_key))) {
     return gst_controller_set_control_source (ctrl, property_name, csource);
   }
   return FALSE;
@@ -238,12 +236,12 @@ gst_object_get_control_source (GObject * object, gchar * property_name)
 {
   GstController *ctrl = NULL;
 
-  g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
+  g_return_val_if_fail (G_IS_OBJECT (object), NULL);
 
-  if ((ctrl = g_object_get_qdata (object, __gst_controller_key))) {
+  if ((ctrl = g_object_get_qdata (object, priv_gst_controller_key))) {
     return gst_controller_get_control_source (ctrl, property_name);
   }
-  return FALSE;
+  return NULL;
 }
 
 /**
@@ -273,7 +271,7 @@ gst_object_get_value_arrays (GObject * object, GstClockTime timestamp,
   g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
   g_return_val_if_fail (GST_CLOCK_TIME_IS_VALID (timestamp), FALSE);
 
-  if ((ctrl = g_object_get_qdata (object, __gst_controller_key))) {
+  if ((ctrl = g_object_get_qdata (object, priv_gst_controller_key))) {
     return gst_controller_get_value_arrays (ctrl, timestamp, value_arrays);
   }
   return (FALSE);
@@ -304,7 +302,7 @@ gst_object_get_value_array (GObject * object, GstClockTime timestamp,
   g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
   g_return_val_if_fail (GST_CLOCK_TIME_IS_VALID (timestamp), FALSE);
 
-  if ((ctrl = g_object_get_qdata (object, __gst_controller_key))) {
+  if ((ctrl = g_object_get_qdata (object, priv_gst_controller_key))) {
     return gst_controller_get_value_array (ctrl, timestamp, value_array);
   }
   return (FALSE);
@@ -336,7 +334,7 @@ gst_object_get_control_rate (GObject * object)
 
   g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
 
-  if ((ctrl = g_object_get_qdata (object, __gst_controller_key))) {
+  if ((ctrl = g_object_get_qdata (object, priv_gst_controller_key))) {
     g_object_get (ctrl, "control-rate", &control_rate, NULL);
   }
   return (control_rate);
@@ -364,7 +362,7 @@ gst_object_set_control_rate (GObject * object, GstClockTime control_rate)
 
   g_return_if_fail (G_IS_OBJECT (object));
 
-  if ((ctrl = g_object_get_qdata (object, __gst_controller_key))) {
+  if ((ctrl = g_object_get_qdata (object, priv_gst_controller_key))) {
     g_object_set (ctrl, "control-rate", control_rate, NULL);
   }
 }
