@@ -21,7 +21,6 @@
  */
 /**
  * SECTION:element-filesrc
- * @short_description: read from arbitrary point in a file
  * @see_also: #GstFileSrc
  *
  * Read data from a file in the local file system.
@@ -36,6 +35,20 @@
 
 #include <stdio.h>
 #include <sys/types.h>
+#ifdef G_OS_WIN32
+#include <io.h>                 /* lseek, open, close, read */
+/* On win32, stat* default to 32 bit; we need the 64-bit
+ * variants, so explicitly define it that way. */
+#define stat __stat64
+#define fstat _fstat64
+#undef lseek
+#define lseek _lseeki64
+#undef off_t
+#define off_t guint64
+/* Prevent stat.h from defining the stat* functions as
+ * _stat*, since we're explicitly overriding that */
+#undef _INC_STAT_INL
+#endif
 #include <sys/stat.h>
 #include <fcntl.h>
 
@@ -47,10 +60,6 @@
 # include <sys/mman.h>
 #endif
 
-#ifdef HAVE_WIN32
-#  include <io.h>               /* lseek, open, close, read */
-#endif
-
 #include <errno.h>
 #include <string.h>
 
@@ -75,6 +84,34 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
 #define O_BINARY (0)
 #endif
 
+/* Copy of glib's g_open due to win32 libc/cross-DLL brokenness: we can't
+ * use the 'file descriptor' opened in glib (and returned from this function)
+ * in this library, as they may have unrelated C runtimes. */
+int
+gst_open (const gchar * filename, int flags, int mode)
+{
+#ifdef G_OS_WIN32
+  wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
+  int retval;
+  int save_errno;
+
+  if (wfilename == NULL) {
+    errno = EINVAL;
+    return -1;
+  }
+
+  retval = _wopen (wfilename, flags, mode);
+  save_errno = errno;
+
+  g_free (wfilename);
+
+  errno = save_errno;
+  return retval;
+#else
+  return open (filename, flags, mode);
+#endif
+}
+
 
 /**********************************************************************
  * GStreamer Default File Source
@@ -118,12 +155,6 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
 GST_DEBUG_CATEGORY_STATIC (gst_file_src_debug);
 #define GST_CAT_DEFAULT gst_file_src_debug
 
-static const GstElementDetails gst_file_src_details =
-GST_ELEMENT_DETAILS ("File Source",
-    "Source/File",
-    "Read from arbitrary point in a file",
-    "Erik Walthinsen <omega@cse.ogi.edu>");
-
 /* FileSrc signals and args */
 enum
 {
@@ -188,10 +219,13 @@ gst_file_src_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
+  gst_element_class_set_details_simple (gstelement_class,
+      "File Source",
+      "Source/File",
+      "Read from arbitrary point in a file",
+      "Erik Walthinsen <omega@cse.ogi.edu>");
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&srctemplate));
-
-  gst_element_class_set_details (gstelement_class, &gst_file_src_details);
 }
 
 static void
@@ -211,18 +245,19 @@ gst_file_src_class_init (GstFileSrcClass * klass)
   g_object_class_install_property (gobject_class, ARG_FD,
       g_param_spec_int ("fd", "File-descriptor",
           "File-descriptor for the file being mmap()d", 0, G_MAXINT, 0,
-          G_PARAM_READABLE));
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_LOCATION,
       g_param_spec_string ("location", "File Location",
-          "Location of the file to read", NULL, G_PARAM_READWRITE));
+          "Location of the file to read", NULL,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_MMAPSIZE,
       g_param_spec_ulong ("mmapsize", "mmap() Block Size",
           "Size in bytes of mmap()d regions", 0, G_MAXULONG, DEFAULT_MMAPSIZE,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_TOUCH,
       g_param_spec_boolean ("touch", "Touch mapped region read data",
           "Touch mmapped data regions to force them to be read from disk",
-          DEFAULT_TOUCH, G_PARAM_READWRITE));
+          DEFAULT_TOUCH, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
    * GstFileSrc:use-mmap
    *
@@ -244,12 +279,12 @@ gst_file_src_class_init (GstFileSrcClass * klass)
   g_object_class_install_property (gobject_class, ARG_USEMMAP,
       g_param_spec_boolean ("use-mmap", "Use mmap to read data",
           "Whether to use mmap() instead of read()",
-          DEFAULT_USEMMAP, G_PARAM_READWRITE));
+          DEFAULT_USEMMAP, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_SEQUENTIAL,
       g_param_spec_boolean ("sequential", "Optimise for sequential mmap access",
           "Whether to use madvise to hint to the kernel that access to "
           "mmap pages will be sequential",
-          DEFAULT_SEQUENTIAL, G_PARAM_READWRITE));
+          DEFAULT_SEQUENTIAL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_file_src_finalize);
 
@@ -302,16 +337,14 @@ gst_file_src_finalize (GObject * object)
 static gboolean
 gst_file_src_set_location (GstFileSrc * src, const gchar * location)
 {
-  /* the element must be stopped in order to do this */
-  GST_STATE_LOCK (src);
-  {
-    GstState state;
+  GstState state;
 
-    state = GST_STATE (src);
-    if (state != GST_STATE_READY && state != GST_STATE_NULL)
-      goto wrong_state;
-  }
-  GST_STATE_UNLOCK (src);
+  /* the element must be stopped in order to do this */
+  GST_OBJECT_LOCK (src);
+  state = GST_STATE (src);
+  if (state != GST_STATE_READY && state != GST_STATE_NULL)
+    goto wrong_state;
+  GST_OBJECT_UNLOCK (src);
 
   g_free (src->filename);
   g_free (src->uri);
@@ -321,6 +354,8 @@ gst_file_src_set_location (GstFileSrc * src, const gchar * location)
     src->filename = NULL;
     src->uri = NULL;
   } else {
+    /* we store the filename as received by the application. On Windoes this
+     * should be UTF8 */
     src->filename = g_strdup (location);
     src->uri = gst_uri_construct ("file", src->filename);
   }
@@ -333,7 +368,7 @@ gst_file_src_set_location (GstFileSrc * src, const gchar * location)
 wrong_state:
   {
     GST_DEBUG_OBJECT (src, "setting location in wrong state");
-    GST_STATE_UNLOCK (src);
+    GST_OBJECT_UNLOCK (src);
     return FALSE;
   }
 }
@@ -447,6 +482,7 @@ struct _GstMmapBufferClass
 static void gst_mmap_buffer_init (GTypeInstance * instance, gpointer g_class);
 static void gst_mmap_buffer_class_init (gpointer g_class, gpointer class_data);
 static void gst_mmap_buffer_finalize (GstMmapBuffer * mmap_buffer);
+static GstBufferClass *mmap_buffer_parent_class = NULL;
 
 static GType
 gst_mmap_buffer_get_type (void)
@@ -478,6 +514,8 @@ gst_mmap_buffer_class_init (gpointer g_class, gpointer class_data)
 {
   GstMiniObjectClass *mini_object_class = GST_MINI_OBJECT_CLASS (g_class);
 
+  mmap_buffer_parent_class = g_type_class_peek_parent (g_class);
+
   mini_object_class->finalize =
       (GstMiniObjectFinalizeFunction) gst_mmap_buffer_finalize;
 }
@@ -526,10 +564,13 @@ gst_mmap_buffer_finalize (GstMmapBuffer * mmap_buffer)
    * guint64 as hex */
   GST_LOG ("unmapped region %08lx+%08lx at %p",
       (gulong) offset, (gulong) size, data);
+
+  GST_MINI_OBJECT_CLASS (mmap_buffer_parent_class)->
+      finalize (GST_MINI_OBJECT (mmap_buffer));
 }
 
 static GstBuffer *
-gst_file_src_map_region (GstFileSrc * src, off_t offset, size_t size,
+gst_file_src_map_region (GstFileSrc * src, off_t offset, gsize size,
     gboolean testonly)
 {
   GstBuffer *buf;
@@ -576,16 +617,16 @@ gst_file_src_map_region (GstFileSrc * src, off_t offset, size_t size,
 mmap_failed:
   {
     if (!testonly) {
-      GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
-          ("mmap (0x%08lx, %d, 0x%llx) failed: %s",
-              (gulong) size, src->fd, offset, g_strerror (errno)));
+      GST_ELEMENT_ERROR (src, RESOURCE, READ, (NULL),
+          ("mmap (0x%08lx, %d, 0x%" G_GINT64_MODIFIER "x) failed: %s",
+              (gulong) size, src->fd, (guint64) offset, g_strerror (errno)));
     }
     return NULL;
   }
 }
 
 static GstBuffer *
-gst_file_src_map_small_region (GstFileSrc * src, off_t offset, size_t size)
+gst_file_src_map_small_region (GstFileSrc * src, off_t offset, gsize size)
 {
   GstBuffer *ret;
   off_t mod;
@@ -601,7 +642,7 @@ gst_file_src_map_small_region (GstFileSrc * src, off_t offset, size_t size)
 
   /* if the offset starts at a non-page boundary, we have to special case */
   if (mod != 0) {
-    size_t mapsize;
+    gsize mapsize;
     off_t mapbase;
     GstBuffer *map;
 
@@ -632,7 +673,7 @@ gst_file_src_create_mmap (GstFileSrc * src, guint64 offset, guint length,
     GstBuffer ** buffer)
 {
   GstBuffer *buf = NULL;
-  size_t readsize, mapsize;
+  gsize readsize, mapsize;
   off_t readend, mapstart, mapend;
   int i;
 
@@ -688,21 +729,21 @@ gst_file_src_create_mmap (GstFileSrc * src, guint64 offset, guint length,
   /* then deal with the case where the read buffer is totally outside */
   if (buf == NULL) {
     /* first check to see if there's a map that covers the right region already */
-    GST_LOG_OBJECT (src, "searching for mapbuf to cover %llu+%d",
-        offset, (int) readsize);
+    GST_LOG_OBJECT (src, "searching for mapbuf to cover %" G_GUINT64_FORMAT
+        "+%d", offset, (int) readsize);
 
     /* if the read buffer crosses a mmap region boundary, create a one-off region */
     if ((offset / src->mapsize) != (readend / src->mapsize)) {
-      GST_LOG_OBJECT (src,
-          "read buf %llu+%d crosses a %d-byte boundary, creating a one-off",
-          offset, (int) readsize, (int) src->mapsize);
+      GST_LOG_OBJECT (src, "read buf %" G_GUINT64_FORMAT "+%d crosses a "
+          "%d-byte boundary, creating a one-off", offset, (int) readsize,
+          (int) src->mapsize);
       buf = gst_file_src_map_small_region (src, offset, readsize);
       if (buf == NULL)
         goto could_not_mmap;
 
       /* otherwise we will create a new mmap region and set it to the default */
     } else {
-      size_t mapsize;
+      gsize mapsize;
 
       off_t nextmap = offset - (offset % src->mapsize);
 
@@ -906,7 +947,8 @@ gst_file_src_start (GstBaseSrc * basesrc)
   GST_INFO_OBJECT (src, "opening file %s", src->filename);
 
   /* open the file */
-  src->fd = open (src->filename, O_RDONLY | O_BINARY);
+  src->fd = gst_open (src->filename, O_RDONLY | O_BINARY, 0);
+
   if (src->fd < 0)
     goto open_failed;
 
@@ -1082,6 +1124,7 @@ gst_file_src_uri_set_uri (GstURIHandler * handler, const gchar * uri)
     return TRUE;
   } else {
     location = gst_uri_get_location (uri);
+    GST_LOG_OBJECT (src, "Location '%s' found from uri '%s'", location, uri);
   }
 
   if (!location)
