@@ -292,8 +292,10 @@ GST_START_TEST (test_message_state_changed_children)
   fail_unless (pending == GST_STATE_VOID_PENDING);
 
   /* wait for async thread to settle down */
+  GST_DEBUG ("waiting for refcount");
   while (GST_OBJECT_REFCOUNT_VALUE (pipeline) > 3)
     THREAD_SWITCH ();
+  GST_DEBUG ("refcount <= 3 now");
 
   /* each object is referenced by a message;
    * base_src is blocked in the push and has an extra refcount.
@@ -456,9 +458,9 @@ GST_START_TEST (test_add_linked)
   sink = gst_element_factory_make ("fakesink", NULL);
   fail_if (sink == NULL, "Could not create fakesink");
 
-  srcpad = gst_element_get_pad (src, "src");
+  srcpad = gst_element_get_static_pad (src, "src");
   fail_unless (srcpad != NULL);
-  sinkpad = gst_element_get_pad (sink, "sink");
+  sinkpad = gst_element_get_static_pad (sink, "sink");
   fail_unless (sinkpad != NULL);
 
   fail_unless (gst_pad_link (srcpad, sinkpad) == GST_PAD_LINK_OK);
