@@ -26,7 +26,9 @@
 GST_START_TEST (test_remove1)
 {
   GstElement *b1, *b2, *src, *sink;
+
   GstPad *srcpad, *sinkpad;
+
   GstPadLinkReturn ret;
 
   b1 = gst_element_factory_make ("pipeline", NULL);
@@ -44,13 +46,13 @@ GST_START_TEST (test_remove1)
   ASSERT_OBJECT_REFCOUNT (b1, "pipeline", 1);
   ASSERT_OBJECT_REFCOUNT (b2, "bin", 1);
 
-  sinkpad = gst_element_get_pad (sink, "sink");
+  sinkpad = gst_element_get_static_pad (sink, "sink");
   gst_element_add_pad (b2, gst_ghost_pad_new ("sink", sinkpad));
   gst_object_unref (sinkpad);
 
-  srcpad = gst_element_get_pad (src, "src");
+  srcpad = gst_element_get_static_pad (src, "src");
   /* get the ghostpad */
-  sinkpad = gst_element_get_pad (b2, "sink");
+  sinkpad = gst_element_get_static_pad (b2, "sink");
 
   ret = gst_pad_link (srcpad, sinkpad);
   fail_unless (ret == GST_PAD_LINK_OK);
@@ -62,7 +64,7 @@ GST_START_TEST (test_remove1)
   ASSERT_OBJECT_REFCOUNT (b2, "bin", 1);
   gst_bin_remove (GST_BIN (b1), b2);
 
-  srcpad = gst_element_get_pad (src, "src");
+  srcpad = gst_element_get_static_pad (src, "src");
   /* pad cannot be linked now */
   fail_if (gst_pad_is_linked (srcpad));
   gst_object_unref (srcpad);
@@ -78,7 +80,9 @@ GST_END_TEST;
 GST_START_TEST (test_remove2)
 {
   GstElement *b1, *b2, *src, *sink;
+
   GstPad *srcpad, *sinkpad;
+
   GstPadLinkReturn ret;
 
   b1 = gst_element_factory_make ("pipeline", NULL);
@@ -92,14 +96,14 @@ GST_START_TEST (test_remove2)
   fail_unless (gst_bin_add (GST_BIN (b1), b2));
   ASSERT_OBJECT_REFCOUNT (src, "src", 1);
 
-  sinkpad = gst_element_get_pad (sink, "sink");
+  sinkpad = gst_element_get_static_pad (sink, "sink");
   gst_element_add_pad (b2, gst_ghost_pad_new ("sink", sinkpad));
   gst_object_unref (sinkpad);
 
-  srcpad = gst_element_get_pad (src, "src");
+  srcpad = gst_element_get_static_pad (src, "src");
   ASSERT_OBJECT_REFCOUNT (srcpad, "srcpad", 2); /* since we got one */
   /* get the ghostpad */
-  sinkpad = gst_element_get_pad (b2, "sink");
+  sinkpad = gst_element_get_static_pad (b2, "sink");
   ASSERT_OBJECT_REFCOUNT (sinkpad, "sinkpad", 2);       /* since we got one */
 
   GST_DEBUG ("linking srcpad and sinkpad");
@@ -116,7 +120,7 @@ GST_START_TEST (test_remove2)
   /* now remove the sink from the bin */
   gst_bin_remove (GST_BIN (b2), sink);
 
-  srcpad = gst_element_get_pad (src, "src");
+  srcpad = gst_element_get_static_pad (src, "src");
   /* pad is still linked to ghostpad */
   fail_if (!gst_pad_is_linked (srcpad));
   ASSERT_OBJECT_REFCOUNT (src, "src", 1);
@@ -151,10 +155,15 @@ GST_END_TEST;
 GST_START_TEST (test_ghost_pads_notarget)
 {
   GstElement *b1, *b2, *sink;
+
   GstPad *srcpad, *sinkpad, *peer;
+
   GstPadLinkReturn ret;
+
   gboolean bret;
+
   GstBus *bus;
+
   GstCaps *caps;
 
   b1 = gst_element_factory_make ("pipeline", NULL);
@@ -172,7 +181,7 @@ GST_START_TEST (test_ghost_pads_notarget)
 
   srcpad = gst_ghost_pad_new_no_target ("src", GST_PAD_SRC);
   fail_unless (srcpad != NULL);
-  sinkpad = gst_element_get_pad (sink, "sink");
+  sinkpad = gst_element_get_static_pad (sink, "sink");
   fail_unless (sinkpad != NULL);
 
   ret = gst_pad_link (srcpad, sinkpad);
@@ -214,7 +223,9 @@ GST_END_TEST;
 GST_START_TEST (test_link)
 {
   GstElement *b1, *b2, *src, *sink;
+
   GstPad *srcpad, *sinkpad, *gpad;
+
   GstPadLinkReturn ret;
 
   b1 = gst_element_factory_make ("pipeline", NULL);
@@ -226,9 +237,9 @@ GST_START_TEST (test_link)
   fail_unless (gst_bin_add (GST_BIN (b1), src));
   fail_unless (gst_bin_add (GST_BIN (b1), b2));
 
-  srcpad = gst_element_get_pad (src, "src");
+  srcpad = gst_element_get_static_pad (src, "src");
   fail_unless (srcpad != NULL);
-  sinkpad = gst_element_get_pad (sink, "sink");
+  sinkpad = gst_element_get_static_pad (sink, "sink");
   fail_unless (sinkpad != NULL);
 
   /* linking in different hierarchies should fail */
@@ -269,7 +280,9 @@ GST_END_TEST;
 GST_START_TEST (test_ghost_pads)
 {
   GstElement *b1, *b2, *src, *i1, *sink;
+
   GstPad *gsink, *gsrc, *gisrc, *gisink, *isink, *isrc, *fsrc, *fsink;
+
   GstStateChangeReturn ret;
 
   b1 = gst_element_factory_make ("pipeline", NULL);
@@ -292,18 +305,18 @@ GST_START_TEST (test_ghost_pads)
   fail_unless (GST_IS_GHOST_PAD (b2->srcpads->data));
   GST_OBJECT_UNLOCK (b2);
 
-  fsrc = gst_element_get_pad (src, "src");
+  fsrc = gst_element_get_static_pad (src, "src");
   fail_unless (fsrc != NULL);
   gsink = GST_PAD (gst_object_ref (b2->sinkpads->data));
   fail_unless (gsink != NULL);
   gsrc = GST_PAD (gst_object_ref (b2->srcpads->data));
   fail_unless (gsrc != NULL);
-  fsink = gst_element_get_pad (sink, "sink");
+  fsink = gst_element_get_static_pad (sink, "sink");
   fail_unless (fsink != NULL);
 
-  isink = gst_element_get_pad (i1, "sink");
+  isink = gst_element_get_static_pad (i1, "sink");
   fail_unless (isink != NULL);
-  isrc = gst_element_get_pad (i1, "src");
+  isrc = gst_element_get_static_pad (i1, "src");
   fail_unless (isrc != NULL);
   gisrc = gst_pad_get_peer (isink);
   fail_unless (gisrc != NULL);
@@ -373,11 +386,17 @@ GST_END_TEST;
 GST_START_TEST (test_ghost_pads_bin)
 {
   GstBin *pipeline;
+
   GstBin *srcbin;
+
   GstBin *sinkbin;
+
   GstElement *src;
+
   GstElement *sink;
+
   GstPad *srcpad, *srcghost, *target;
+
   GstPad *sinkpad, *sinkghost;
 
   pipeline = GST_BIN (gst_pipeline_new ("pipe"));
@@ -393,14 +412,14 @@ GST_START_TEST (test_ghost_pads_bin)
 
   src = gst_element_factory_make ("fakesrc", "src");
   gst_bin_add (srcbin, src);
-  srcpad = gst_element_get_pad (src, "src");
+  srcpad = gst_element_get_static_pad (src, "src");
   srcghost = gst_ghost_pad_new ("src", srcpad);
   gst_object_unref (srcpad);
   gst_element_add_pad (GST_ELEMENT (srcbin), srcghost);
 
   sink = gst_element_factory_make ("fakesink", "sink");
   gst_bin_add (sinkbin, sink);
-  sinkpad = gst_element_get_pad (sink, "sink");
+  sinkpad = gst_element_get_static_pad (sink, "sink");
   sinkghost = gst_ghost_pad_new ("sink", sinkpad);
   gst_object_unref (sinkpad);
   gst_element_add_pad (GST_ELEMENT (sinkbin), sinkghost);
@@ -443,10 +462,15 @@ block_callback (GstPad * pad, gboolean blocked, gpointer user_data)
 GST_START_TEST (test_ghost_pads_block)
 {
   GstBin *pipeline;
+
   GstBin *srcbin;
+
   GstElement *src;
+
   GstPad *srcpad;
+
   GstPad *srcghost;
+
   BlockData block_data;
 
   pipeline = GST_BIN (gst_pipeline_new ("pipeline"));
@@ -456,7 +480,7 @@ GST_START_TEST (test_ghost_pads_block)
 
   src = gst_element_factory_make ("fakesrc", "src");
   gst_bin_add (srcbin, src);
-  srcpad = gst_element_get_pad (src, "src");
+  srcpad = gst_element_get_static_pad (src, "src");
   srcghost = gst_ghost_pad_new ("src", srcpad);
   gst_element_add_pad (GST_ELEMENT (srcbin), srcghost);
   gst_object_unref (srcpad);
@@ -484,10 +508,15 @@ GST_END_TEST;
 GST_START_TEST (test_ghost_pads_probes)
 {
   GstBin *pipeline;
+
   GstBin *srcbin;
+
   GstElement *src;
+
   GstPad *srcpad;
+
   GstPad *srcghost;
+
   BlockData block_data;
 
   pipeline = GST_BIN (gst_pipeline_new ("pipeline"));
@@ -497,7 +526,7 @@ GST_START_TEST (test_ghost_pads_probes)
 
   src = gst_element_factory_make ("fakesrc", "src");
   gst_bin_add (srcbin, src);
-  srcpad = gst_element_get_pad (src, "src");
+  srcpad = gst_element_get_static_pad (src, "src");
   srcghost = gst_ghost_pad_new ("src", srcpad);
   gst_element_add_pad (GST_ELEMENT (srcbin), srcghost);
   gst_object_unref (srcpad);
@@ -525,20 +554,31 @@ GST_END_TEST;
 GST_START_TEST (test_ghost_pads_new_from_template)
 {
   GstPad *sinkpad, *ghostpad;
+
   GstPadTemplate *padtempl, *ghosttempl;
+
   GstCaps *padcaps, *ghostcaps, *newcaps;
+  GstCaps *copypadcaps;
 
   padcaps = gst_caps_from_string ("some/caps");
   fail_unless (padcaps != NULL);
   ghostcaps = gst_caps_from_string ("some/caps;some/other-caps");
   fail_unless (ghostcaps != NULL);
 
+  copypadcaps = gst_caps_copy (padcaps);
   padtempl = gst_pad_template_new ("padtempl", GST_PAD_SINK,
-      GST_PAD_ALWAYS, padcaps);
+      GST_PAD_ALWAYS, copypadcaps);
   fail_unless (padtempl != NULL);
   ghosttempl = gst_pad_template_new ("ghosttempl", GST_PAD_SINK,
       GST_PAD_ALWAYS, ghostcaps);
 
+  /* FIXME : We should not have to unref those caps, but due to 
+   * a bug in gst_pad_template_new() not stealing the refcount of
+   * the given caps we have to. */
+  gst_caps_unref (ghostcaps);
+  gst_caps_unref (copypadcaps);
+
+
   sinkpad = gst_pad_new_from_template (padtempl, "sinkpad");
   fail_unless (sinkpad != NULL);
 
@@ -554,6 +594,12 @@ GST_START_TEST (test_ghost_pads_new_from_template)
   fail_unless (gst_caps_is_equal (newcaps, padcaps));
   gst_caps_unref (newcaps);
   gst_caps_unref (padcaps);
+
+  gst_object_unref (sinkpad);
+  gst_object_unref (ghostpad);
+
+  gst_object_unref (padtempl);
+  gst_object_unref (ghosttempl);
 }
 
 GST_END_TEST;
@@ -561,19 +607,31 @@ GST_END_TEST;
 GST_START_TEST (test_ghost_pads_new_no_target_from_template)
 {
   GstPad *sinkpad, *ghostpad;
+
   GstPadTemplate *padtempl, *ghosttempl;
+
   GstCaps *padcaps, *ghostcaps, *newcaps;
+  GstCaps *copypadcaps, *copyghostcaps;
 
   padcaps = gst_caps_from_string ("some/caps");
   fail_unless (padcaps != NULL);
   ghostcaps = gst_caps_from_string ("some/caps;some/other-caps");
   fail_unless (ghostcaps != NULL);
 
+  copypadcaps = gst_caps_copy (padcaps);
+  copyghostcaps = gst_caps_copy (ghostcaps);
+
   padtempl = gst_pad_template_new ("padtempl", GST_PAD_SINK,
-      GST_PAD_ALWAYS, padcaps);
+      GST_PAD_ALWAYS, copypadcaps);
   fail_unless (padtempl != NULL);
   ghosttempl = gst_pad_template_new ("ghosttempl", GST_PAD_SINK,
-      GST_PAD_ALWAYS, ghostcaps);
+      GST_PAD_ALWAYS, copyghostcaps);
+
+  /* FIXME : We should not have to unref those caps, but due to 
+   * a bug in gst_pad_template_new() not stealing the refcount of
+   * the given caps we have to. */
+  gst_caps_unref (copyghostcaps);
+  gst_caps_unref (copypadcaps);
 
   sinkpad = gst_pad_new_from_template (padtempl, "sinkpad");
   fail_unless (sinkpad != NULL);
@@ -598,14 +656,23 @@ GST_START_TEST (test_ghost_pads_new_no_target_from_template)
   fail_unless (gst_caps_is_equal (newcaps, padcaps));
   gst_caps_unref (newcaps);
 
+  gst_object_unref (sinkpad);
+  gst_object_unref (ghostpad);
+
+  gst_object_unref (padtempl);
+  gst_object_unref (ghosttempl);
+
+  gst_caps_unref (padcaps);
+  gst_caps_unref (ghostcaps);
 }
 
 GST_END_TEST;
 
-Suite *
+static Suite *
 gst_ghost_pad_suite (void)
 {
   Suite *s = suite_create ("GstGhostPad");
+
   TCase *tc_chain = tcase_create ("ghost pad tests");
 
   suite_add_tcase (s, tc_chain);
