@@ -435,6 +435,137 @@ GST_START_TEST (segment_seek_reverse)
 
 GST_END_TEST;
 
+/* mess with the segment structure in the bytes format */
+GST_START_TEST (segment_seek_rate)
+{
+  GstSegment segment;
+  gboolean update;
+
+  gst_segment_init (&segment, GST_FORMAT_BYTES);
+
+  /* configure segment to rate 2.0, format does not matter when we don't specify
+   * a start or stop position. */
+  gst_segment_set_seek (&segment, 2.0,
+      GST_FORMAT_UNDEFINED,
+      GST_SEEK_FLAG_NONE,
+      GST_SEEK_TYPE_NONE, -1, GST_SEEK_TYPE_NONE, -1, &update);
+  fail_unless (segment.format == GST_FORMAT_BYTES);
+  fail_unless (segment.start == 0);
+  fail_unless (segment.stop == -1);
+  fail_unless (segment.rate == 2.0);
+  fail_unless (update == FALSE);
+
+  /* 0 is the same in all formats and should not fail */
+  gst_segment_set_seek (&segment, 2.0,
+      GST_FORMAT_TIME, GST_SEEK_FLAG_NONE,
+      GST_SEEK_TYPE_SET, 0, GST_SEEK_TYPE_NONE, -1, &update);
+  fail_unless (segment.format == GST_FORMAT_BYTES);
+
+  /* set to -1 means start from 0 */
+  gst_segment_set_seek (&segment, 2.0,
+      GST_FORMAT_TIME, GST_SEEK_FLAG_NONE,
+      GST_SEEK_TYPE_SET, -1, GST_SEEK_TYPE_NONE, -1, &update);
+  fail_unless (segment.format == GST_FORMAT_BYTES);
+  fail_unless (segment.start == 0);
+
+  gst_segment_set_seek (&segment, 2.0,
+      GST_FORMAT_TIME, GST_SEEK_FLAG_NONE,
+      GST_SEEK_TYPE_CUR, 0, GST_SEEK_TYPE_NONE, -1, &update);
+
+  gst_segment_set_seek (&segment, 2.0,
+      GST_FORMAT_TIME, GST_SEEK_FLAG_NONE,
+      GST_SEEK_TYPE_END, 0, GST_SEEK_TYPE_NONE, -1, &update);
+
+  /* -1 for end is fine too in all formats */
+  gst_segment_set_seek (&segment, 2.0,
+      GST_FORMAT_TIME, GST_SEEK_FLAG_NONE,
+      GST_SEEK_TYPE_NONE, -1, GST_SEEK_TYPE_SET, -1, &update);
+
+  /* 0 as relative end is fine too */
+  gst_segment_set_seek (&segment, 2.0,
+      GST_FORMAT_TIME, GST_SEEK_FLAG_NONE,
+      GST_SEEK_TYPE_NONE, -1, GST_SEEK_TYPE_CUR, 0, &update);
+
+  gst_segment_set_seek (&segment, 2.0,
+      GST_FORMAT_TIME, GST_SEEK_FLAG_NONE,
+      GST_SEEK_TYPE_NONE, -1, GST_SEEK_TYPE_END, 0, &update);
+
+  /* set a real stop position, this must happen in bytes */
+  gst_segment_set_seek (&segment, 3.0,
+      GST_FORMAT_BYTES,
+      GST_SEEK_FLAG_NONE,
+      GST_SEEK_TYPE_NONE, -1, GST_SEEK_TYPE_SET, 100, &update);
+  fail_unless (segment.format == GST_FORMAT_BYTES);
+  fail_unless (segment.start == 0);
+  fail_unless (segment.stop == 100);
+  fail_unless (segment.rate == 3.0);
+  /* no seek should happen, we just updated the stop position in forward
+   * playback mode.*/
+  fail_unless (update == FALSE);
+
+  /* 0 as relative end is fine too */
+  gst_segment_set_seek (&segment, 2.0,
+      GST_FORMAT_TIME, GST_SEEK_FLAG_NONE,
+      GST_SEEK_TYPE_NONE, -1, GST_SEEK_TYPE_CUR, 0, &update);
+  fail_unless (segment.stop == 100);
+
+  gst_segment_set_seek (&segment, 2.0,
+      GST_FORMAT_TIME, GST_SEEK_FLAG_NONE,
+      GST_SEEK_TYPE_NONE, -1, GST_SEEK_TYPE_END, 0, &update);
+  fail_unless (segment.stop == 100);
+
+  /* -1 for end is fine too in all formats */
+  gst_segment_set_seek (&segment, 2.0,
+      GST_FORMAT_TIME, GST_SEEK_FLAG_NONE,
+      GST_SEEK_TYPE_NONE, -1, GST_SEEK_TYPE_SET, -1, &update);
+  fail_unless (segment.stop == -1);
+
+  /* set some duration, stop -1 END seeks will now work with the
+   * duration, if the formats match */
+  gst_segment_set_duration (&segment, GST_FORMAT_BYTES, 200);
+  fail_unless (segment.duration == 200);
+
+  /* seek to end in any format with 0 should set the stop to the
+   * duration */
+  gst_segment_set_seek (&segment, 2.0,
+      GST_FORMAT_TIME, GST_SEEK_FLAG_NONE,
+      GST_SEEK_TYPE_NONE, -1, GST_SEEK_TYPE_END, 0, &update);
+  fail_unless (segment.stop == 200);
+  fail_unless (segment.duration == 200);
+
+  /* subtract 100 from the end */
+  gst_segment_set_seek (&segment, 2.0,
+      GST_FORMAT_BYTES, GST_SEEK_FLAG_NONE,
+      GST_SEEK_TYPE_NONE, -1, GST_SEEK_TYPE_END, -100, &update);
+  fail_unless (segment.stop == 100);
+  fail_unless (segment.duration == 200);
+
+  /* add 100 to the duration, this should be clamped to the duration */
+  gst_segment_set_seek (&segment, 2.0,
+      GST_FORMAT_BYTES, GST_SEEK_FLAG_NONE,
+      GST_SEEK_TYPE_NONE, -1, GST_SEEK_TYPE_END, 100, &update);
+  fail_unless (segment.stop == 200);
+  fail_unless (segment.duration == 200);
+
+  /* add 300 to the start, this should be clamped to the duration */
+  gst_segment_set_seek (&segment, 2.0,
+      GST_FORMAT_BYTES, GST_SEEK_FLAG_NONE,
+      GST_SEEK_TYPE_CUR, 300, GST_SEEK_TYPE_END, 0, &update);
+  fail_unless (segment.start == 200);
+  fail_unless (segment.stop == 200);
+  fail_unless (segment.duration == 200);
+
+  /* subtract 300 from the start, this should be clamped to 0 */
+  gst_segment_set_seek (&segment, 2.0,
+      GST_FORMAT_BYTES, GST_SEEK_FLAG_NONE,
+      GST_SEEK_TYPE_CUR, -300, GST_SEEK_TYPE_END, 0, &update);
+  fail_unless (segment.start == 0);
+  fail_unless (segment.stop == 200);
+  fail_unless (segment.duration == 200);
+}
+
+GST_END_TEST;
+
 /* mess with the segment structure in the bytes format */
 GST_START_TEST (segment_newsegment_open)
 {
@@ -1493,7 +1624,30 @@ GST_START_TEST (segment_newsegment_accum2)
 
 GST_END_TEST;
 
-Suite *
+GST_START_TEST (segment_copy)
+{
+  GstSegment *copy;
+  GstSegment segment = { 0.0, };
+
+  /* this is a boxed type copy function, we support copying NULL */
+  fail_unless (gst_segment_copy (NULL) == NULL);
+
+  gst_segment_init (&segment, GST_FORMAT_TIME);
+
+  gst_segment_set_newsegment_full (&segment, FALSE, -1.0, 1.0,
+      GST_FORMAT_TIME, 0, 200, 0);
+
+  copy = gst_segment_copy (&segment);
+  fail_unless (copy != NULL);
+  /* we inited the struct on the stack to zeroes, so direct comparison should
+   * be ok here despite the padding field and regardless of implementation */
+  fail_unless (memcmp (copy, &segment, sizeof (GstSegment)) == 0);
+  gst_segment_free (copy);
+}
+
+GST_END_TEST;
+
+static Suite *
 gst_segment_suite (void)
 {
   Suite *s = suite_create ("GstSegment");
@@ -1505,6 +1659,7 @@ gst_segment_suite (void)
   tcase_add_test (tc_chain, segment_seek_nosize);
   tcase_add_test (tc_chain, segment_seek_size);
   tcase_add_test (tc_chain, segment_seek_reverse);
+  tcase_add_test (tc_chain, segment_seek_rate);
   tcase_add_test (tc_chain, segment_newsegment_open);
   tcase_add_test (tc_chain, segment_newsegment_closed);
   tcase_add_test (tc_chain, segment_newsegment_streamtime);
@@ -1514,6 +1669,7 @@ gst_segment_suite (void)
   tcase_add_test (tc_chain, segment_newsegment_runningtime);
   tcase_add_test (tc_chain, segment_newsegment_accum);
   tcase_add_test (tc_chain, segment_newsegment_accum2);
+  tcase_add_test (tc_chain, segment_copy);
 
   return s;
 }
