@@ -21,6 +21,27 @@
 
 #include <gst/check/gstcheck.h>
 
+/* see if the defines make sense */
+GST_START_TEST (test_range)
+{
+  GstClockTime time, time2;
+
+  time = GST_SECOND;
+  fail_unless (time == G_GUINT64_CONSTANT (1000000000));
+
+  time2 = time / 1000;
+  fail_unless (time2 == 1000000);
+  fail_unless (time2 == GST_MSECOND);
+  fail_unless (time2 == GST_TIME_AS_USECONDS (time));
+
+  time2 = time / 1000000;
+  fail_unless (time2 == 1000);
+  fail_unless (time2 == GST_USECOND);
+  fail_unless (time2 == GST_TIME_AS_MSECONDS (time));
+}
+
+GST_END_TEST;
+
 GST_START_TEST (test_signedness)
 {
   GstClockTime time[] = { 0, 1, G_MAXUINT64 / GST_SECOND };
@@ -36,22 +57,23 @@ GST_START_TEST (test_signedness)
   }
 }
 
-GST_END_TEST
+GST_END_TEST;
+
 #define TIME_UNIT (GST_SECOND / 5)
-    static void
+static void
 gst_clock_debug (GstClock * clock)
 {
   GstClockTime time;
 
   time = gst_clock_get_time (clock);
-  g_message ("Clock info: time %" G_GUINT64_FORMAT "\n", time);
+  GST_DEBUG ("Clock info: time %" GST_TIME_FORMAT, GST_TIME_ARGS (time));
 }
 
 static gboolean
 ok_callback (GstClock * clock, GstClockTime time,
     GstClockID id, gpointer user_data)
 {
-  g_message ("unlocked async id %p\n", id);
+  GST_LOG ("unlocked async id %p", id);
   return FALSE;
 }
 
@@ -59,7 +81,7 @@ static gboolean
 error_callback (GstClock * clock, GstClockTime time,
     GstClockID id, gpointer user_data)
 {
-  g_message ("unlocked unscheduled async id %p, this is wrong\n", id);
+  GST_WARNING ("unlocked unscheduled async id %p, this is wrong", id);
   fail_if (TRUE);
 
   return FALSE;
@@ -69,10 +91,10 @@ static gboolean
 store_callback (GstClock * clock, GstClockTime time,
     GstClockID id, gpointer user_data)
 {
-  GstClockID *store_id = user_data;
+  GList **list = user_data;
 
-  g_message ("unlocked async id %p\n", id);
-  *store_id = id;
+  GST_DEBUG ("unlocked async id %p", id);
+  *list = g_list_append (*list, id);
   return FALSE;
 }
 
@@ -104,53 +126,58 @@ GST_START_TEST (test_single_shot)
   id = gst_clock_new_single_shot_id (clock, base + TIME_UNIT);
   fail_unless (id != NULL, "Could not create single shot id");
 
-  g_message ("waiting one second\n");
+  GST_DEBUG ("waiting one time unit");
   result = gst_clock_id_wait (id, NULL);
   gst_clock_debug (clock);
-  fail_unless (result == GST_CLOCK_OK, "Waiting did not return OK");
+  fail_unless (result == GST_CLOCK_OK, "Waiting did not return OK (result=%d)",
+      result);
+  fail_unless (gst_clock_get_time (clock) > (base + TIME_UNIT),
+      "target time has not been reached");
 
-  g_message ("waiting in the past\n");
+  GST_DEBUG ("waiting in the past");
   result = gst_clock_id_wait (id, NULL);
   gst_clock_debug (clock);
-  fail_unless (result == GST_CLOCK_EARLY, "Waiting did not return EARLY");
+  fail_unless (result == GST_CLOCK_EARLY,
+      "Waiting did not return EARLY(result=%d)", result);
   gst_clock_id_unref (id);
 
   id = gst_clock_new_single_shot_id (clock, base + 2 * TIME_UNIT);
-  g_message ("waiting one second async id %p\n", id);
+  GST_DEBUG ("waiting one second async id %p", id);
   result = gst_clock_id_wait_async (id, ok_callback, NULL);
   gst_clock_id_unref (id);
   fail_unless (result == GST_CLOCK_OK, "Waiting did not return OK");
   g_usleep (TIME_UNIT / (2 * 1000));
 
   id = gst_clock_new_single_shot_id (clock, base + 5 * TIME_UNIT);
-  g_message ("waiting one second async, with cancel on id %p\n", id);
+  GST_DEBUG ("waiting one second async, with cancel on id %p", id);
   result = gst_clock_id_wait_async (id, error_callback, NULL);
   fail_unless (result == GST_CLOCK_OK, "Waiting did not return OK");
   g_usleep (TIME_UNIT / (2 * 1000));
-  g_message ("cancel id %p after half a time unit\n", id);
+  GST_DEBUG ("cancel id %p after half a time unit", id);
   gst_clock_id_unschedule (id);
   gst_clock_id_unref (id);
-  g_message ("canceled id %p\n", id);
+  GST_DEBUG ("canceled id %p", id);
 
-  g_message ("waiting multiple one second async, with cancel\n");
+  GST_DEBUG ("waiting multiple one second async, with cancel");
   id = gst_clock_new_single_shot_id (clock, base + 5 * TIME_UNIT);
   id2 = gst_clock_new_single_shot_id (clock, base + 6 * TIME_UNIT);
-  g_message ("waiting id %p\n", id);
+  GST_DEBUG ("waiting id %p", id);
   result = gst_clock_id_wait_async (id, ok_callback, NULL);
   fail_unless (result == GST_CLOCK_OK, "Waiting did not return OK");
   gst_clock_id_unref (id);
-  g_message ("waiting id %p\n", id2);
+  GST_DEBUG ("waiting id %p", id2);
   result = gst_clock_id_wait_async (id2, error_callback, NULL);
   fail_unless (result == GST_CLOCK_OK, "Waiting did not return OK");
   g_usleep (TIME_UNIT / (2 * 1000));
-  g_message ("cancel id %p after half a time unit\n", id2);
+  GST_DEBUG ("cancel id %p after half a time unit", id2);
   gst_clock_id_unschedule (id2);
-  g_message ("canceled id %p\n", id2);
+  GST_DEBUG ("canceled id %p", id2);
   gst_clock_id_unref (id2);
   g_usleep (TIME_UNIT / (2 * 1000));
 }
 
-GST_END_TEST
+GST_END_TEST;
+
 GST_START_TEST (test_periodic_shot)
 {
   GstClock *clock;
@@ -168,22 +195,22 @@ GST_START_TEST (test_periodic_shot)
   id = gst_clock_new_periodic_id (clock, base + TIME_UNIT, TIME_UNIT / 2);
   fail_unless (id != NULL, "Could not create periodic id");
 
-  g_message ("waiting one time unit\n");
+  GST_DEBUG ("waiting one time unit");
   result = gst_clock_id_wait (id, NULL);
   gst_clock_debug (clock);
   fail_unless (result == GST_CLOCK_OK, "Waiting did not return OK");
 
-  g_message ("waiting for the next\n");
+  GST_DEBUG ("waiting for the next");
   result = gst_clock_id_wait (id, NULL);
   gst_clock_debug (clock);
   fail_unless (result == GST_CLOCK_OK, "Waiting did not return OK");
 
-  g_message ("waiting for the next async %p\n", id);
+  GST_DEBUG ("waiting for the next async %p", id);
   result = gst_clock_id_wait_async (id, ok_callback, NULL);
   fail_unless (result == GST_CLOCK_OK, "Waiting did not return OK");
   g_usleep (TIME_UNIT / (2 * 1000));
 
-  g_message ("waiting some more for the next async %p\n", id);
+  GST_DEBUG ("waiting some more for the next async %p", id);
   result = gst_clock_id_wait_async (id, ok_callback, NULL);
   fail_unless (result == GST_CLOCK_OK, "Waiting did not return OK");
   g_usleep (TIME_UNIT / (2 * 1000));
@@ -191,12 +218,12 @@ GST_START_TEST (test_periodic_shot)
   id2 = gst_clock_new_periodic_id (clock, base + TIME_UNIT, TIME_UNIT / 2);
   fail_unless (id2 != NULL, "Could not create second periodic id");
 
-  g_message ("waiting some more for another async %p\n", id2);
+  GST_DEBUG ("waiting some more for another async %p", id2);
   result = gst_clock_id_wait_async (id2, ok_callback, NULL);
   fail_unless (result == GST_CLOCK_OK, "Waiting did not return OK");
   g_usleep (TIME_UNIT / (2 * 1000));
 
-  g_message ("unschedule %p\n", id);
+  GST_DEBUG ("unschedule %p", id);
   gst_clock_id_unschedule (id);
 
   /* entry cannot be used again */
@@ -212,11 +239,13 @@ GST_START_TEST (test_periodic_shot)
   gst_clock_id_unref (id);
 }
 
-GST_END_TEST
+GST_END_TEST;
+
 GST_START_TEST (test_async_order)
 {
   GstClock *clock;
-  GstClockID id1, id2, last_id = NULL;
+  GstClockID id1, id2;
+  GList *cb_list = NULL, *next;
   GstClockTime base;
   GstClockReturn result;
 
@@ -228,20 +257,28 @@ GST_START_TEST (test_async_order)
 
   id1 = gst_clock_new_single_shot_id (clock, base + 2 * TIME_UNIT);
   id2 = gst_clock_new_single_shot_id (clock, base + 1 * TIME_UNIT);
-  result = gst_clock_id_wait_async (id1, store_callback, &last_id);
+  result = gst_clock_id_wait_async (id1, store_callback, &cb_list);
   fail_unless (result == GST_CLOCK_OK, "Waiting did not return OK");
   g_usleep (TIME_UNIT / (2 * 1000));
-  result = gst_clock_id_wait_async (id2, store_callback, &last_id);
+  result = gst_clock_id_wait_async (id2, store_callback, &cb_list);
   fail_unless (result == GST_CLOCK_OK, "Waiting did not return OK");
   g_usleep (TIME_UNIT / 1000);
-  fail_unless (last_id == id2, "Expected notification for id2 to come first");
+  /* at this point at least one of the timers should have timed out */
+  fail_unless (cb_list != NULL, "expected notification");
+  fail_unless (cb_list->data == id2,
+      "Expected notification for id2 to come first");
   g_usleep (TIME_UNIT / 1000);
-  fail_unless (last_id == id1, "Missing notification for id1");
+  /* now both should have timed out */
+  next = g_list_next (cb_list);
+  fail_unless (next != NULL, "expected second notification");
+  fail_unless (next->data == id1, "Missing notification for id1");
   gst_clock_id_unref (id1);
   gst_clock_id_unref (id2);
+  g_list_free (cb_list);
 }
 
-GST_END_TEST
+GST_END_TEST;
+
 GST_START_TEST (test_periodic_multi)
 {
   GstClock *clock;
@@ -284,13 +321,15 @@ GST_START_TEST (test_periodic_multi)
   fail_unless (gst_clock_get_time (clock) < base + 4 * TIME_UNIT);
 }
 
-GST_END_TEST
+GST_END_TEST;
+
 GST_START_TEST (test_diff)
 {
-  GstClockTime time1[] =
-      { 0, -1, 0, 1, 2 * GST_SECOND, -GST_SECOND, -GST_SECOND };
+  GstClockTime time1[] = { 0, (GstClockTime) - 1, 0, 1, 2 * GST_SECOND,
+    (GstClockTime) - GST_SECOND, (GstClockTime) - GST_SECOND
+  };
   GstClockTime time2[] =
-      { 0, 1, 1, 0, 1 * GST_SECOND, -GST_SECOND, GST_SECOND };
+      { 0, 1, 1, 0, 1 * GST_SECOND, (GstClockTime) - GST_SECOND, GST_SECOND };
   GstClockTimeDiff d[] = { 0, 2, 1, -1, -GST_SECOND, 0, 2 * GST_SECOND };
   guint i;
 
@@ -298,19 +337,93 @@ GST_START_TEST (test_diff)
     fail_if (d[i] != GST_CLOCK_DIFF (time1[i], time2[i]));
   }
 }
-GST_END_TEST Suite *
+
+GST_END_TEST;
+
+/* test if a blocking wait, unblocked by an async entry continues to be
+ * scheduled */
+typedef struct
+{
+  GstClock *clock;
+  GstClockID id;
+  GstClockTimeDiff jitter;
+  GstClockReturn ret;
+} MixedInfo;
+
+static gpointer
+mixed_thread (MixedInfo * info)
+{
+  info->ret = gst_clock_id_wait (info->id, &info->jitter);
+  return NULL;
+}
+
+static gboolean
+mixed_async_cb (GstClock * clock, GstClockTime time,
+    GstClockID id, gpointer user_data)
+{
+  return TRUE;
+}
+
+GST_START_TEST (test_mixed)
+{
+  GThread *thread;
+  GError *error = NULL;
+  MixedInfo info;
+  GstClockTime base;
+  GstClockID id;
+
+  info.clock = gst_system_clock_obtain ();
+  fail_unless (info.clock != NULL,
+      "Could not create instance of GstSystemClock");
+
+  /* get current time of the clock as base time */
+  base = gst_clock_get_time (info.clock);
+
+  /* create entry to wait for 1 second */
+  info.id = gst_clock_new_single_shot_id (info.clock, base + GST_SECOND);
+
+  /* make and start an entry that is scheduled every 10ms */
+  id = gst_clock_new_periodic_id (info.clock, base, 10 * GST_MSECOND);
+
+  /* start waiting for the entry */
+  thread = g_thread_create ((GThreadFunc) mixed_thread, &info, TRUE, &error);
+  fail_unless (error == NULL, "error creating thread");
+  fail_unless (thread != NULL, "Could not create thread");
+
+  /* wait half a second so we are sure to be in the thread */
+  g_usleep (G_USEC_PER_SEC / 2);
+
+  /* start scheduling the entry */
+  gst_clock_id_wait_async (id, mixed_async_cb, NULL);
+
+  /* wait for thread to finish */
+  g_thread_join (thread);
+  /* entry must have timed out correctly */
+  fail_unless (info.ret == GST_CLOCK_OK, "clock return was %d", info.ret);
+
+  gst_clock_id_unschedule (id);
+  gst_clock_id_unref (id);
+  gst_clock_id_unref (info.id);
+  gst_object_unref (info.clock);
+}
+
+GST_END_TEST;
+
+static Suite *
 gst_systemclock_suite (void)
 {
   Suite *s = suite_create ("GstSystemClock");
   TCase *tc_chain = tcase_create ("waiting");
 
   suite_add_tcase (s, tc_chain);
+  tcase_add_test (tc_chain, test_range);
   tcase_add_test (tc_chain, test_signedness);
   tcase_add_test (tc_chain, test_single_shot);
   tcase_add_test (tc_chain, test_periodic_shot);
   tcase_add_test (tc_chain, test_periodic_multi);
   tcase_add_test (tc_chain, test_async_order);
   tcase_add_test (tc_chain, test_diff);
+  tcase_add_test (tc_chain, test_mixed);
 
   return s;
 }
